//
//  DXAspectControllerBasic.m
//  DiceX
//
//  Created by Andrew Merenbach on 6/23/07.
//  Copyright 2007 Andrew Merenbach. All rights reserved.
//

#import "DXAspectControllerBasic.h"

NSString *AMCountOfDiceKey = @"countOfDice";
NSString *AMSidesPerDieKey = @"sidesPerDie";
NSString *AMWillRollAsPercentileKey = @"willRollAsPercentile";
NSString *AMValueForPercentileEqualsMaxKey = @"valueForPercentileEqualsMax";

@implementation DXAspectControllerBasic

- (id)init {
	if (nil != (self = [super initWithNibName:@"BasicAspect"])) {
		NSBundle *bundle = [NSBundle mainBundle];
		NSString *path = [bundle pathForResource:@"DefaultListValues" ofType:@"plist"];
		NSDictionary *dict = [NSDictionary dictionaryWithContentsOfFile:path];
		
		[countOfDiceComboBox addItemsWithObjectValues:[dict objectForKey:@"dieCountComboBox"]];
		[sidesPerDieComboBox addItemsWithObjectValues:[dict objectForKey:@"dieSidesComboBox"]];
		
		[self setCountOfDice:0];	//AM_countOfDice = 1;
		[self setSidesPerDie:0];	//AM_sidesPerDie = 2;
		[self setWillRollAsPercentile:NO];	//AM_rollAsPercentile = NO;
		[self setPercentileZeroEqualsMax:YES];	//AM_percentileZeroEqualsMax = YES;
		[self setValueForPercentileZeroEqualsMax:0];
	}
	return self;
}

- (void)awakeFromNib {
	[self updateZeroMaxTextField];
}

- (NSString *)label {
	return @"Basic Configuration";
}

- (NSString *)identifier {
	return @"Basic";
}

- (void)modifiedPercentileRoll:(id)sender {
	// this NSOnState check is not, currently, strictly necessary
	if (NSOnState == [sender state]) {
		[self setSidesPerDie:10];
	}
	[self updateZeroMaxTextField];

	[[NSNotificationCenter defaultCenter] postNotificationName:@"AMModifyPercentileRoll" object:self];
}

- (void)updateZeroMaxTextField {
	if ([self willRollAsPercentile]) {
		unsigned int dieCount = [self countOfDice];
		unsigned int power = (unsigned int)pow((double)10, (double)dieCount);
		[self setValueForPercentileZeroEqualsMax:power];
	} else {
		[self setValueForPercentileZeroEqualsMax:0];
	}
}

- (void)loadValuesFromDefaults {
	[super loadValuesFromDefaults];
	
	id userDefaults = [NSUserDefaults standardUserDefaults];

	[self setCountOfDice:[userDefaults integerForKey:AMCountOfDiceKey]];
	[self setSidesPerDie:[userDefaults integerForKey:AMSidesPerDieKey]];
	[self setWillRollAsPercentile:[userDefaults boolForKey:AMWillRollAsPercentileKey]];
	[self setValueForPercentileZeroEqualsMax:[userDefaults integerForKey:AMValueForPercentileEqualsMaxKey]];
}

- (void)saveValuesToDefaults {
	[super saveValuesToDefaults];
	
	id userDefaults = [NSUserDefaults standardUserDefaults];

	[userDefaults setInteger:[self countOfDice] forKey:AMCountOfDiceKey];	
	[userDefaults setInteger:[self sidesPerDie] forKey:AMSidesPerDieKey];
	[userDefaults setBool:[self willRollAsPercentile] forKey:AMWillRollAsPercentileKey];
	[userDefaults setInteger:[self valueForPercentileZeroEqualsMax] forKey:AMValueForPercentileEqualsMaxKey];
}

- (NSString *)summary {
	NSMutableArray *sentences = [NSMutableArray array];	//WithObject:[super summary];
	NSString *sentence;

	if ([self willRollAsPercentile]) {
	
		sentence = [[NSString alloc] initWithFormat:@"Make a %id%i roll as percentile.", [self countOfDice], [self sidesPerDie]];
		[sentences addObject:sentence];
		[sentence release];
		
		if ([self percentileZeroEqualsMax]) {
			sentence = [[NSString alloc] initWithFormat:@"A roll of 0 will count as %i.", [self valueForPercentileZeroEqualsMax]];
			[sentences addObject:sentence];
			[sentence release];
		} else {
			sentence = [[NSString alloc] initWithFormat:@"A roll of 0 will still count as 0."];
			[sentences addObject:sentence];
			[sentence release];
		}
		
	} else {

		sentence = [[NSString alloc] initWithFormat:@"Make a %id%i roll.", [self countOfDice], [self sidesPerDie]];
		[sentences addObject:sentence];
		[sentence release];
	}
	
	return [sentences componentsJoinedByString:AMDoubleSpace];

}

@end

@implementation DXAspectControllerBasic (Accessors)

- (int)countOfDice {
	return AM_countOfDice;
}

- (void)setCountOfDice:(int)anInt {
	AM_countOfDice = anInt;
	[[NSNotificationCenter defaultCenter] postNotificationName:@"AMModifiedSettings" object:self];
	[self updateZeroMaxTextField];
}

- (int)sidesPerDie {
	return AM_sidesPerDie;
}

- (void)setSidesPerDie:(int)anInt {
	AM_sidesPerDie = anInt;
	[[NSNotificationCenter defaultCenter] postNotificationName:@"AMModifiedSettings" object:self];
}

- (BOOL)willRollAsPercentile {
	return AM_rollAsPercentile;
}

- (void)setWillRollAsPercentile:(BOOL)flag {
	AM_rollAsPercentile = flag;
	[[NSNotificationCenter defaultCenter] postNotificationName:@"AMModifiedSettings" object:self];
}

- (BOOL)percentileZeroEqualsMax {
	return AM_percentileZeroEqualsMax;
}

- (void)setPercentileZeroEqualsMax:(BOOL)flag {
	AM_percentileZeroEqualsMax = flag;
	[[NSNotificationCenter defaultCenter] postNotificationName:@"AMModifiedSettings" object:self];
}

- (unsigned int)valueForPercentileZeroEqualsMax {
	return AM_valueForPercentileZeroEqualsMax;
}

- (void)setValueForPercentileZeroEqualsMax:(unsigned int)anInt {
	AM_valueForPercentileZeroEqualsMax = anInt;
}

@end

