//
//  DXAspectControllerDrops.m
//  DiceX
//
//  Created by Andrew Merenbach on 6/23/07.
//  Copyright 2007 Andrew Merenbach. All rights reserved.
//

#import "DXAspectControllerDrops.h"
#import "DXAspectControllerBasic.h"

NSString *AMWillDropLowestValueKey = @"willDropLowestValue";
NSString *AMWillDropHighestValueKey = @"willDropHighestValue";
NSString *AMLowestDropValueKey = @"lowestDropValue";
NSString *AMHighestDropValueKey = @"highestDropValue";


@implementation DXAspectControllerDrops

- (id)init {
	if (nil != (self = [super initWithNibName:@"DropsAspect"])) {
		[self setWillDropLowestValue:NO];	//AM_willDropLowestValue = NO;
		[self setWillDropHighestValue:NO];	//AM_willDropHighestValue = NO;
		[self setLowestDropValue:0];	//AM_lowestDropValue = 0;
		[self setHighestDropValue:0];	//AM_highestDropValue = 0;
		[self setWillRollAsPercentile:NO];	//AM_willRollAsPercentile = NO;
		
		[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(percentileRollApplyChanges:) name:@"AMModifyPercentileRoll" object:nil];
	}
	return self;
}

- (void)dealloc {
	[[NSNotificationCenter defaultCenter] removeObserver:self name:@"AMModifyPercentileRoll" object:nil];

	[super dealloc];
}

- (NSString *)label {
	return @"Roll Dropping";
}

- (NSString *)identifier {
	return @"Drops";
}

- (void)percentileRollApplyChanges:(NSNotification *)aNotification {
	BOOL percentile = [[aNotification object] willRollAsPercentile];
	[self setWillRollAsPercentile:percentile];
	
	if (percentile) {
		[self setWillDropLowestValue:NO];
		[self setWillDropHighestValue:NO];
		
		//[self setLowestDropValue:0];
		//[self setHighestDropValue:0];
	}
}

- (void)loadValuesFromDefaults {
	[super loadValuesFromDefaults];
	
	id userDefaults = [NSUserDefaults standardUserDefaults];
		
	[self setWillDropLowestValue:[userDefaults boolForKey:AMWillDropLowestValueKey]];
	[self setWillDropHighestValue:[userDefaults boolForKey:AMWillDropHighestValueKey]];
	[self setLowestDropValue:[userDefaults integerForKey:AMLowestDropValueKey]];
	[self setHighestDropValue:[userDefaults integerForKey:AMHighestDropValueKey]];
	
	//[self setWillRollAsPercentile:[userDefaults boolForKey:AMWillRollAsPercentileKey]];
}

- (void)saveValuesToDefaults {
	[super saveValuesToDefaults];

	id userDefaults = [NSUserDefaults standardUserDefaults];
	
	[userDefaults setBool:[self willDropLowestValue] forKey:AMWillDropLowestValueKey];
	[userDefaults setBool:[self willDropHighestValue] forKey:AMWillDropHighestValueKey];
	[userDefaults setInteger:[self lowestDropValue] forKey:AMLowestDropValueKey];
	[userDefaults setInteger:[self highestDropValue] forKey:AMHighestDropValueKey];
	
	//[userDefaults setBool:[self willRollAsPercentile] forKey:AMWillRollAsPercentileKey];
}

- (NSString *)summary {
	NSMutableArray *sentences = [NSMutableArray array];	//WithObject:[super summary];
	NSString *sentence;
	
	NSString *rollWithPluralCheck;
	int dropValue;
	
	if (![self willRollAsPercentile]) {
		
		if ([self willDropLowestValue]) {
			dropValue = [self lowestDropValue];
			rollWithPluralCheck = ((dropValue  != 1) ? @"rolls" : @"roll");
			
			sentence = [[NSString alloc] initWithFormat:@"The lowest %i %@ will be dropped.", dropValue, rollWithPluralCheck];
			[sentences addObject:sentence];
			[sentence release];
		}
		
		if ([self willDropHighestValue]) {
			dropValue = [self highestDropValue];
			rollWithPluralCheck = ((dropValue != 1) ? @"rolls" : @"roll");
			
			sentence = [[NSString alloc] initWithFormat:@"The highest %i %@ will be dropped.", dropValue, rollWithPluralCheck];
			[sentences addObject:sentence];
			[sentence release];
		}

	}
	
	return [sentences componentsJoinedByString:AMDoubleSpace];
}

@end

@implementation DXAspectControllerDrops (DropsAccessors)

- (BOOL)willDropLowestValue {
	return AM_willDropLowestValue;
}

- (void)setWillDropLowestValue:(BOOL)flag {
	AM_willDropLowestValue = flag;
	[[NSNotificationCenter defaultCenter] postNotificationName:@"AMModifiedSettings" object:self];
}

- (BOOL)willDropHighestValue {
	return AM_willDropHighestValue;
}

- (void)setWillDropHighestValue:(BOOL)flag {
	AM_willDropHighestValue = flag;
	[[NSNotificationCenter defaultCenter] postNotificationName:@"AMModifiedSettings" object:self];
}

- (int)lowestDropValue {
	return AM_lowestDropValue;
}

- (void)setLowestDropValue:(int)anInt {
	AM_lowestDropValue = anInt;
	[[NSNotificationCenter defaultCenter] postNotificationName:@"AMModifiedSettings" object:self];
}

- (int)highestDropValue {
	return AM_highestDropValue;
}

- (void)setHighestDropValue:(int)anInt {
	AM_highestDropValue = anInt;
	[[NSNotificationCenter defaultCenter] postNotificationName:@"AMModifiedSettings" object:self];
}

@end

@implementation DXAspectControllerDrops (PercentileDiceExtensions)

- (BOOL)willRollAsPercentile {
	return AM_willRollAsPercentile;
}

- (void)setWillRollAsPercentile:(BOOL)flag {
	AM_willRollAsPercentile = flag;
}

@end
