//
//  DXAspectControllerMain.m
//  DiceX
//
//  Created by Andrew Merenbach on 6/23/07.
//  Copyright 2007 Andrew Merenbach. All rights reserved.
//

#import "DXAspectControllerMain.h"
#import "DXDieGroup.h"

NSString *AMLogRollsKey = @"logRolls";
NSString *AMAutoscrollLogEntriesKey = @"logAutoscrollEntries";
NSString *AMAutoExpandLogEntriesKey = @"logAutoExpandEntries";
NSString *AMRollRepetitionsKey = @"rollRepetitions";


@implementation DXAspectControllerMain

- (id)init {
	if (nil != (self = [super initWithNibName:@"MainAspect"])) {
		//[self setResult:0];
		[self setLogRolls:YES];
		[self setAutoscrollLogEntries:YES];
		[self setAutoExpandLogEntries:NO];
		[self setRollRepetitions:1];
		//[self setNumberOfNewEntries:0];
		
		NSZone *thisZone = [self zone];
		
		masterOrderHint = 0;
		
		NSSortDescriptor *sortDescriptor = [[NSSortDescriptor allocWithZone:thisZone] initWithKey:@"orderHint" ascending:YES];
		[self willChangeValueForKey:@"resultSortDescriptors"];
		AM_resultSortDescriptors = [[NSArray allocWithZone:thisZone] initWithObjects:sortDescriptor, nil];
		[self didChangeValueForKey:@"resultSortDescriptors"];
		[sortDescriptor release];
		
		[self setProcessingRolls:NO];
		
		[self setSaving:NO];
		
		[self setPersistentStoreCoordinatorRef:nil];
		
		[self setTitleForRollButton:@"Roll"];
		
		//mainManagedObjectContext = [[NSManagedObjectContext allocWithZone:thisZone] init];
	}
	return self;
}

- (void)dealloc {
	[self setPersistentStoreCoordinatorRef:nil];
	
	/* we can't do [self setResultSortDescriptors:nil] because
		that method, deliberately, doesn't do anything at all */
	[AM_resultSortDescriptors release];
	AM_resultSortDescriptors = nil;
	
	[AM_titleForRollButton release];
	AM_titleForRollButton = nil;
	
	//[mainManagedObjectContext release];
	//mainManagedObjectContext = nil;
	
	[super dealloc];
}

/*- (void)awakeFromNib {
	//[mainManagedObjectContext setPersistentStoreCoordinator:[[[NSApp delegate] managedObjectContext] persistentStoreCoordinator]];
	psCoordinator = [[[NSApp delegate] managedObjectContext] persistentStoreCoordinator];
}*/

- (NSString *)label {
	return @"Main Interface";
}

- (NSString *)identifier {
	return @"Main";
}

/*- (void)roll:(id)sender {
	
}*/

- (void)loadValuesFromDefaults {
	[super loadValuesFromDefaults];

	id userDefaults = [NSUserDefaults standardUserDefaults];
	
	[self setLogRolls:[userDefaults boolForKey:AMLogRollsKey]];
	[self setAutoscrollLogEntries:[userDefaults boolForKey:AMAutoscrollLogEntriesKey]];
	[self setAutoExpandLogEntries:[userDefaults boolForKey:AMAutoExpandLogEntriesKey]];
	
	[self setRollRepetitions:[userDefaults integerForKey:AMRollRepetitionsKey]];
}

- (void)saveValuesToDefaults {
	[super saveValuesToDefaults];
	
	id userDefaults = [NSUserDefaults standardUserDefaults];
	
	[userDefaults setBool:[self logRolls] forKey:AMLogRollsKey];
	[userDefaults setBool:[self autoscrollLogEntries] forKey:AMAutoscrollLogEntriesKey];
	[userDefaults setBool:[self autoExpandLogEntries] forKey:AMAutoExpandLogEntriesKey];

	[userDefaults setInteger:[self rollRepetitions] forKey:AMRollRepetitionsKey];
}

- (void)clearRolls {
	unsigned count = [[rollArrayController arrangedObjects] count];
	if (count > 0) {
		NSMutableIndexSet *indexSet = [[NSMutableIndexSet alloc] initWithIndexesInRange:NSMakeRange(0, count)];
		[rollArrayController performSelectorOnMainThread:@selector(removeObjectsAtArrangedObjectIndexes:) withObject:indexSet waitUntilDone:YES];
		[indexSet release];
		
		masterOrderHint = 0;
	}
}

- (void)addRolls:(NSArray *)rolls {
	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
	
	//NSManagedObjectContext *context = [[NSApp delegate] managedObjectContext];

	NSManagedObjectContext *context = [[NSManagedObjectContext alloc] init];
	[context setPersistentStoreCoordinator:[self persistentStoreCoordinatorRef]];
	[[context undoManager] disableUndoRegistration];
		
	NSEntityDescription *rollEntity = [NSEntityDescription entityForName:@"RollResult" inManagedObjectContext:context];

	//NSMutableArray *results = [[NSMutableArray alloc] initWithCapacity:[rolls count]];
	
	NSManagedObject *result;
	
	
	NSEnumerator *e = [rolls objectEnumerator];
	id roll;
	
	while ([self isProcessingRolls] && (roll = [e nextObject])) {
		if (result = [[NSManagedObject alloc] initWithEntity:rollEntity insertIntoManagedObjectContext:context]) {
			++masterOrderHint;
			
			[result setValue:[NSNumber numberWithUnsignedInt:masterOrderHint] forKey:@"orderHint"];
			[result setValue:[NSNumber numberWithUnsignedInt:masterOrderHint] forKey:@"rollNumber"];
			[result setValue:[NSNumber numberWithInt:[roll total]] forKey:@"rollResult"];
			[result setValue:[NSNumber numberWithDouble:[roll percentage]] forKey:@"rollPercentage"];
			//[results addObject:result];
			//[rollArrayController addObject:result];
			[result release];
		}
	}
	
	//[rollArrayController performSelectorOnMainThread:@selector(addObjects:) withObject:results waitUntilDone:YES];
	//[results release];
	//[self performSelectorOnMainThread:@selector(finishAddingRoll) withObject:nil waitUntilDone:YES];
	
	[self setSaving:YES];
	NSError *error;
	if (![context save:&error]) {
		[NSApp performSelectorOnMainThread:@selector(presentError:) withObject:error waitUntilDone:YES];
	}
	[self setSaving:NO];
	
	[self performSelectorOnMainThread:@selector(finishAddingRolls) withObject:nil waitUntilDone:YES];
	[self setProcessingRolls:NO];
	
	[context release];
	
	[pool release];
}

- (void)finishAddingRolls {
	NSError *error = nil;
	if (![rollArrayController fetchWithRequest:nil merge:YES error:&error]) {
		[[NSApplication sharedApplication] presentError:error];
	}
}

/*- (BOOL)fetchRollEntries {
	BOOL success = NO;

	NSManagedObjectContext *moc = [[NSApp delegate] managedObjectContext];
	
	NSError *fetchError = nil;
	NSFetchRequest *fetchRequest = [[NSFetchRequest alloc] init];
	@try {
		NSEntityDescription *entity = [NSEntityDescription entityForName:@"RollResult"
			inManagedObjectContext:moc];
		[fetchRequest setEntity:entity];
		
		//NSPredicate *predicate = [NSPredicate predicateWithFormat:@"parent == nil"];
		//[fetchRequest setPredicate:predicate];
		
		[fetchRequest setSortDescriptors:[self resultSortDescriptors]];

		success = [rollArrayController fetchWithRequest:fetchRequest merge:YES error:&fetchError];
	} @finally {
		[fetchRequest release];
	}

	if (fetchError != nil) {
		[[NSApp delegate] presentError:fetchError];
	}
	
	return success;
}*/

@end

@implementation DXAspectControllerMain (Accessors)

- (BOOL)isSaving {
	return AM_saving;
}

- (void)setSaving:(BOOL)flag {
	AM_saving = flag;
}

- (BOOL)isProcessingRolls {
	return AM_processingRolls;
}

- (void)setProcessingRolls:(BOOL)flag {
	AM_processingRolls = flag;
	[[NSNotificationCenter defaultCenter] postNotificationName:@"AMModifiedRollProcessingStatus" object:nil];
}

/*- (int)result {
	return AM_result;
}

- (void)setResult:(int)anInt {
	AM_result = anInt;
}*/

- (BOOL)logRolls {
	return AM_logRolls;
}

- (void)setLogRolls:(BOOL)flag {
	AM_logRolls = flag;
}

- (BOOL)autoscrollLogEntries {
	return AM_autoscrollLogEntries;
}

- (void)setAutoscrollLogEntries:(BOOL)flag {
	AM_autoscrollLogEntries = flag;
}

- (BOOL)autoExpandLogEntries {
	return AM_autoExpandLogEntries;
}

- (void)setAutoExpandLogEntries:(BOOL)flag {
	AM_autoExpandLogEntries = flag;
}

- (int)rollRepetitions {
	return AM_rollRepetitions;
}

- (void)setRollRepetitions:(int)anInt {
	AM_rollRepetitions = anInt;
	[[NSNotificationCenter defaultCenter] postNotificationName:@"AMModifiedSettings" object:self];
}

- (NSArray *)resultSortDescriptors {
	return [[AM_resultSortDescriptors retain] autorelease];
}

- (void)setResultSortDescriptors:(NSArray *)anArray {
	/* do nothing */
	/*@synchronized(AM_resultSortDescriptors) {
		if (anArray != AM_resultSortDescriptors) {
			[AM_resultSortDescriptors release];
			AM_resultSortDescriptors = anArray;
		}
	}*/
}

- (NSPersistentStoreCoordinator *)persistentStoreCoordinatorRef {
	return AM_psCoordinator;
}

- (void)setPersistentStoreCoordinatorRef:(NSPersistentStoreCoordinator *)coordinator {
	AM_psCoordinator = coordinator;
}

- (NSString *)titleForRollButton {
	return AM_titleForRollButton;
}

- (void)setTitleForRollButton:(NSString *)aString {
	@synchronized(AM_titleForRollButton) {
		if (aString != AM_titleForRollButton) {
			[AM_titleForRollButton release];
			AM_titleForRollButton = [aString retain];
		}
	}
}

@end
