//
//  DXAspectControllerModifiers.m
//  DiceX
//
//  Created by Andrew Merenbach on 6/23/07.
//  Copyright 2007 Andrew Merenbach. All rights reserved.
//

#import "DXAspectControllerModifiers.h"
#import "DXAspectControllerBasic.h"

NSString *AMWillApplyPerDieBonusValueKey = @"willApplyPerDieBonusValue";
NSString *AMWillApplyPerDiePenaltyValueKey = @"willApplyPerDiePenaltyValue";
NSString *AMWillApplyPerDieMultiplierValueKey = @"willApplyPerDieMultiplierValue";

NSString *AMWillApplyToTotalBonusValueKey = @"willApplyToTotalBonusValue";
NSString *AMWillApplyToTotalPenaltyValueKey = @"willApplyToTotalPenaltyValue";
NSString *AMWillApplyToTotalMultiplierValueKey = @"willApplyToTotalMultiplierValue";

NSString *AMPerDieBonusValueKey = @"bonusValuePerDie";
NSString *AMPerDiePenaltyValueKey = @"penaltyValuePerDie";
NSString *AMPerDieMultiplierValueKey = @"multiplierValuePerDie";

NSString *AMToTotalBonusValueKey = @"bonusValueToTotal";
NSString *AMToTotalPenaltyValueKey = @"penaltyValueToTotal";
NSString *AMToTotalMultiplierValueKey = @"multiplierValueToTotal";


@implementation DXAspectControllerModifiers

- (id)init {
	if (nil != (self = [super initWithNibName:@"ModifiersAspect"])) {
		[self setWillApplyPerDieBonusValue:NO];	//AM_willApplyPerDieBonusValue = NO;
		[self setWillApplyPerDiePenaltyValue:NO];	//AM_willApplyPerDiePenaltyValue = NO;
		[self setWillApplyPerDieMultiplierValue:NO];
		
		[self setWillApplyToTotalBonusValue:NO];	//AM_willApplyToTotalBonusValue = NO;
		[self setWillApplyToTotalPenaltyValue:NO];	//AM_willApplyToTotalPenaltyValue = NO;
		[self setWillApplyToTotalMultiplierValue:NO];
		
		[self setPerDieBonusValue:0];	//AM_perDieBonusValue = 0;
		[self setPerDiePenaltyValue:0];	//AM_perDiePenaltyValue = 0;
		[self setPerDieMultiplierValue:0];
		
		[self setToTotalBonusValue:0];	//AM_toTotalBonusValue = 0;
		[self setToTotalPenaltyValue:0];	//AM_toTotalPenaltyValue = 0;
		[self setToTotalMultiplierValue:0];
		
		[self setWillRollAsPercentile:NO];	//AM_willRollAsPercentile = NO;
		
		[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(percentileRollApplyChanges:) name:@"AMModifyPercentileRoll" object:nil];
	}
	return self;
}

- (void)dealloc {
	[[NSNotificationCenter defaultCenter] removeObserver:self name:@"AMModifyPercentileRoll" object:nil];
	
	[super dealloc];
}

- (NSString *)label {
	return @"Modifiers";
}

- (NSString *)identifier {
	return @"Modifiers";
}

- (void)percentileRollApplyChanges:(NSNotification *)aNotification {
	BOOL percentile = [[aNotification object] willRollAsPercentile];
	[self setWillRollAsPercentile:percentile];
	
	if (percentile) {
		[self setWillApplyPerDieBonusValue:NO];
		[self setWillApplyPerDiePenaltyValue:NO];
	}
}

- (void)loadValuesFromDefaults {
	[super loadValuesFromDefaults];
	
	id userDefaults = [NSUserDefaults standardUserDefaults];
	
	[self setWillApplyPerDieBonusValue:[userDefaults boolForKey:AMWillApplyPerDieBonusValueKey]];
	[self setWillApplyPerDiePenaltyValue:[userDefaults boolForKey:AMWillApplyPerDiePenaltyValueKey]];
	[self setWillApplyPerDieMultiplierValue:[userDefaults boolForKey:AMWillApplyPerDieMultiplierValueKey]];

	[self setWillApplyToTotalBonusValue:[userDefaults boolForKey:AMWillApplyToTotalBonusValueKey]];
	[self setWillApplyToTotalPenaltyValue:[userDefaults boolForKey:AMWillApplyToTotalPenaltyValueKey]];
	[self setWillApplyToTotalMultiplierValue:[userDefaults boolForKey:AMWillApplyToTotalMultiplierValueKey]];

	[self setPerDieBonusValue:[userDefaults integerForKey:AMPerDieBonusValueKey]];
	[self setPerDiePenaltyValue:[userDefaults integerForKey:AMPerDiePenaltyValueKey]];
	[self setPerDieMultiplierValue:[userDefaults integerForKey:AMPerDieMultiplierValueKey]];
	
	[self setToTotalBonusValue:[userDefaults integerForKey:AMToTotalBonusValueKey]];
	[self setToTotalPenaltyValue:[userDefaults integerForKey:AMToTotalPenaltyValueKey]];
	[self setToTotalMultiplierValue:[userDefaults integerForKey:AMToTotalMultiplierValueKey]];

	//[self setWillRollAsPercentile:[userDefaults boolForKey:AMWillRollAsPercentileKey]];
}

- (void)saveValuesToDefaults {
	[super saveValuesToDefaults];
	
	id userDefaults = [NSUserDefaults standardUserDefaults];
	
	[userDefaults setBool:[self willApplyPerDieBonusValue] forKey:AMWillApplyPerDieBonusValueKey];
	[userDefaults setBool:[self willApplyPerDiePenaltyValue] forKey:AMWillApplyPerDiePenaltyValueKey];
	[userDefaults setBool:[self willApplyToTotalBonusValue] forKey:AMWillApplyToTotalBonusValueKey];
	[userDefaults setBool:[self willApplyToTotalPenaltyValue] forKey:AMWillApplyToTotalPenaltyValueKey];
	
	[userDefaults setInteger:[self perDieBonusValue] forKey:AMPerDieBonusValueKey];
	[userDefaults setInteger:[self perDiePenaltyValue] forKey:AMPerDiePenaltyValueKey];
	[userDefaults setInteger:[self toTotalBonusValue] forKey:AMToTotalBonusValueKey];
	[userDefaults setInteger:[self toTotalPenaltyValue] forKey:AMToTotalPenaltyValueKey];
	
	//[userDefaults setBool:[self willRollAsPercentile] forKey:AMWillRollAsPercentileKey];
}

- (NSString *)summary {
	NSMutableArray *sentences = [NSMutableArray array];	//WithObject:[super summary];
	NSString *sentence;
	
	if ([self willRollAsPercentile]) {

		if ([self willApplyToTotalBonusValue]) {
			sentence = [[NSString alloc] initWithFormat:@"A bonus of %i will be added to the total.", [self toTotalBonusValue]];
			[sentences addObject:sentence];
			[sentence release];
		}

		if ([self willApplyToTotalPenaltyValue]) {
			sentence = [[NSString alloc] initWithFormat:@"A penalty of %i will be subtracted from the total.", [self toTotalPenaltyValue]];
			[sentences addObject:sentence];
			[sentence release];
		}
		
		if ([self willApplyToTotalMultiplierValue]) {
			sentence = [[NSString alloc] initWithFormat:@"A multiplier of %i will be applied to the total after the bonus and penalty.", [self toTotalMultiplierValue]];
			[sentences addObject:sentence];
			[sentence release];
		}

	} else {

		if ([self willApplyPerDieBonusValue]) {
			sentence = [[NSString alloc] initWithFormat:@"A bonus of %i will be added to each die.", [self perDieBonusValue]];
			[sentences addObject:sentence];
			[sentence release];
		}
		
		if ([self willApplyPerDiePenaltyValue]) {
			sentence = [[NSString alloc] initWithFormat:@"A penalty of %i will be subtracted from each die.", [self perDiePenaltyValue]];
			[sentences addObject:sentence];
			[sentence release];
		}
		
		if ([self willApplyPerDieMultiplierValue]) {
			sentence = [[NSString alloc] initWithFormat:@"A multiplier of %i will be applied to each die after the bonus and penalty.", [self perDieMultiplierValue]];
			[sentences addObject:sentence];
			[sentence release];
		}
		
		if ([self willApplyToTotalBonusValue]) {
			sentence = [[NSString alloc] initWithFormat:@"A bonus of %i will be added to the total.", [self toTotalBonusValue]];
			[sentences addObject:sentence];
			[sentence release];
		}
		
		if ([self willApplyToTotalPenaltyValue]) {
			sentence = [[NSString alloc] initWithFormat:@"A penalty of %i will be subtracted from the total.", [self toTotalPenaltyValue]];
			[sentences addObject:sentence];
			[sentence release];
		}
		
		if ([self willApplyToTotalMultiplierValue]) {
			sentence = [[NSString alloc] initWithFormat:@"A multiplier of %i will be applied to the total after the bonus and penalty.", [self toTotalMultiplierValue]];
			[sentences addObject:sentence];
			[sentence release];
		}

	}
	
	return [sentences componentsJoinedByString:AMDoubleSpace];
}

@end

@implementation DXAspectControllerModifiers (InputAccessors)

- (BOOL)willApplyPerDieBonusValue {
	return AM_willApplyPerDieBonusValue;
}

- (void)setWillApplyPerDieBonusValue:(BOOL)flag {
	AM_willApplyPerDieBonusValue = flag;
	[[NSNotificationCenter defaultCenter] postNotificationName:@"AMModifiedSettings" object:self];
}

- (BOOL)willApplyPerDiePenaltyValue {
	return AM_willApplyPerDiePenaltyValue;
}

- (void)setWillApplyPerDiePenaltyValue:(BOOL)flag {
	AM_willApplyPerDiePenaltyValue = flag;
	[[NSNotificationCenter defaultCenter] postNotificationName:@"AMModifiedSettings" object:self];
}

- (BOOL)willApplyPerDieMultiplierValue {
	return AM_willApplyPerDieMultiplierValue;
}

- (void)setWillApplyPerDieMultiplierValue:(BOOL)flag {
	AM_willApplyPerDieMultiplierValue = flag;
	[[NSNotificationCenter defaultCenter] postNotificationName:@"AMModifiedSettings" object:self];
}

- (BOOL)willApplyToTotalBonusValue {
	return AM_willApplyToTotalBonusValue;
}

- (void)setWillApplyToTotalBonusValue:(BOOL)flag {
	AM_willApplyToTotalBonusValue = flag;
	[[NSNotificationCenter defaultCenter] postNotificationName:@"AMModifiedSettings" object:self];
}

- (BOOL)willApplyToTotalPenaltyValue {
	return AM_willApplyToTotalPenaltyValue;
}

- (void)setWillApplyToTotalPenaltyValue:(BOOL)flag {
	AM_willApplyToTotalPenaltyValue = flag;
	[[NSNotificationCenter defaultCenter] postNotificationName:@"AMModifiedSettings" object:self];
}

- (BOOL)willApplyToTotalMultiplierValue {
	return AM_willApplyToTotalMultiplierValue;
}

- (void)setWillApplyToTotalMultiplierValue:(BOOL)flag {
	AM_willApplyToTotalMultiplierValue = flag;
	[[NSNotificationCenter defaultCenter] postNotificationName:@"AMModifiedSettings" object:self];
}

- (int)perDieBonusValue {
	return AM_perDieBonusValue;
}

- (void)setPerDieBonusValue:(int)anInt {
	AM_perDieBonusValue = anInt;
	[[NSNotificationCenter defaultCenter] postNotificationName:@"AMModifiedSettings" object:self];
}

- (int)perDiePenaltyValue {
	return AM_perDiePenaltyValue;
}

- (void)setPerDiePenaltyValue:(int)anInt {
	AM_perDiePenaltyValue = anInt;
	[[NSNotificationCenter defaultCenter] postNotificationName:@"AMModifiedSettings" object:self];
}

- (int)perDieMultiplierValue {
	return AM_perDieMultiplierValue;
}

- (void)setPerDieMultiplierValue:(int)anInt {
	AM_perDieMultiplierValue = anInt;
	[[NSNotificationCenter defaultCenter] postNotificationName:@"AMModifiedSettings" object:self];
}

- (int)toTotalBonusValue {
	return AM_toTotalBonusValue;
}

- (void)setToTotalBonusValue:(int)anInt {
	AM_toTotalBonusValue = anInt;
	[[NSNotificationCenter defaultCenter] postNotificationName:@"AMModifiedSettings" object:self];
}

- (int)toTotalPenaltyValue {
	return AM_toTotalPenaltyValue;
}

- (void)setToTotalPenaltyValue:(int)anInt {
	AM_toTotalPenaltyValue = anInt;
	[[NSNotificationCenter defaultCenter] postNotificationName:@"AMModifiedSettings" object:self];
}

- (int)toTotalMultiplierValue {
	return AM_toTotalMultiplierValue;
}

- (void)setToTotalMultiplierValue:(int)anInt {
	AM_toTotalMultiplierValue = anInt;
	[[NSNotificationCenter defaultCenter] postNotificationName:@"AMModifiedSettings" object:self];
}

@end

@implementation DXAspectControllerModifiers (PercentileDiceExtensions)

- (BOOL)willRollAsPercentile {
	return AM_willRollAsPercentile;
}

- (void)setWillRollAsPercentile:(BOOL)flag {
	AM_willRollAsPercentile = flag;
}

@end
