//
//  DXAspectControllerRerolling.h
//  DiceX
//
//  Created by Andrew Merenbach on 6/23/07.
//  Copyright 2007 Andrew Merenbach. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import "DXAspectController.h"

extern NSString *AMWillRerollIndividualDiceUnderValueKey;
extern NSString *AMWillRerollIndividualDiceOverValueKey;
extern NSString *AMRerollIndividualDiceUnderValueKey;
extern NSString *AMRerollIndividualDiceOverValueKey;
extern NSString *AMRerollIndividualDicePrecedenceKey;

extern NSString *AMWillRerollPercentileDiceUnderValueKey;
extern NSString *AMWillRerollPercentileDiceOverValueKey;
extern NSString *AMRerollPercentileDiceUnderValueKey;
extern NSString *AMRerollPercentileDiceOverValueKey;
extern NSString *AMRerollPercentileDicePrecedenceKey;


typedef enum _AMRerollPrecedence {
	AMRerollBeforeModifiers = 0,
	AMRerollAfterModifiers = 1,
} AMRerollPrecedence;

@interface DXAspectControllerRerolling : DXAspectController {
@private

	/* rerolling individual dice */
	BOOL AM_willRerollIndividualDiceUnderValue;
	BOOL AM_willRerollIndividualDiceOverValue;
	
	int AM_rerollIndividualDiceUnderValue;
	int AM_rerollIndividualDiceOverValue;
	
	AMRerollPrecedence AM_rerollIndividualDicePrecedence;
	BOOL AM_rerollingIndividualDiceOptionsEnabled;

	/* rerolling percentile dice */
	BOOL AM_willRollPercentileUnderValue;
	BOOL AM_willRollAsPercentileOverValue;
	
	int AM_rerollPercentileDiceUnderValue;
	int AM_rerollPercentileDiceOverValue;
	
	AMRerollPrecedence AM_rerollPercentileDicePrecedence;
	
	BOOL AM_willRollAsPercentile;
	BOOL AM_rerollingPercentileDiceOptionsEnabled;
}

- (void)percentileRollApplyChanges:(NSNotification *)aNotification;

@end

@interface DXAspectControllerRerolling (RerollingIndividualDice)

- (BOOL)willRerollIndividualDiceUnderValue;
- (void)setWillRerollIndividualDiceUnderValue:(BOOL)flag;
- (BOOL)willRerollIndividualDiceOverValue;
- (void)setWillRerollIndividualDiceOverValue:(BOOL)flag;

- (int)rerollIndividualDiceUnderValue;
- (void)setRerollIndividualDiceUnderValue:(int)anInt;
- (int)rerollIndividualDiceOverValue;
- (void)setRerollIndividualDiceOverValue:(int)anInt;

- (AMRerollPrecedence)rerollIndividualDicePrecedence;
- (void)setRerollIndividualDicePrecedence:(AMRerollPrecedence)aCondition;

- (BOOL)rerollingIndividualDiceOptionsEnabled;
- (void)setRerollingIndividualDiceOptionsEnabled:(BOOL)flag;
- (void)updateRerollingIndividualDiceOptionsEnabled;

@end

@interface DXAspectControllerRerolling (PercentileRerolling)

- (BOOL)willRerollPercentileDiceUnderValue;
- (void)setWillRerollPercentileDiceUnderValue:(BOOL)flag;
- (BOOL)willRerollPercentileDiceOverValue;
- (void)setWillRerollPercentileDiceOverValue:(BOOL)flag;

- (int)rerollPercentileDiceUnderValue;
- (void)setRerollPercentileDiceUnderValue:(int)anInt;
- (int)rerollPercentileDiceOverValue;
- (void)setRerollPercentileDiceOverValue:(int)anInt;

- (AMRerollPrecedence)rerollPercentileDicePrecedence;
- (void)setRerollPercentileDicePrecedence:(AMRerollPrecedence)aCondition;

- (BOOL)rerollingPercentileDiceOptionsEnabled;
- (void)setRerollingPercentileDiceOptionsEnabled:(BOOL)flag;
- (void)updateRerollingPercentileDiceOptionsEnabled;

@end

@interface DXAspectControllerRerolling (PercentileDiceExtensions)

- (BOOL)willRollAsPercentile;
- (void)setWillRollAsPercentile:(BOOL)flag;

@end
