//
//  DXAspectControllerRerolling.m
//  DiceX
//
//  Created by Andrew Merenbach on 6/23/07.
//  Copyright 2007 Andrew Merenbach. All rights reserved.
//

#import "DXAspectControllerRerolling.h"
#import "DXAspectControllerBasic.h"

NSString *AMWillRerollIndividualDiceUnderValueKey = @"willRerollIndividualDiceUnderValue";
NSString *AMWillRerollIndividualDiceOverValueKey = @"willRerollIndividualDiceOverValue";
NSString *AMRerollIndividualDiceUnderValueKey = @"rerollIndividualDiceUnderValue";
NSString *AMRerollIndividualDiceOverValueKey = @"rerollIndividualDiceOverValue";
NSString *AMRerollIndividualDicePrecedenceKey = @"rerollIndividualDicePrecedence";

NSString *AMWillRerollPercentileDiceUnderValueKey = @"willRerollPercentileDiceUnderValue";
NSString *AMWillRerollPercentileDiceOverValueKey = @"willRerollPercentileDiceOverValue";
NSString *AMRerollPercentileDiceUnderValueKey = @"rerollPercentileDiceUnderValue";
NSString *AMRerollPercentileDiceOverValueKey = @"rerollPercentileDiceOverValue";
NSString *AMRerollPercentileDicePrecedenceKey = @"rerollPercentileDicePrecedence";


@implementation DXAspectControllerRerolling

- (id)init {
	if (nil != (self = [super initWithNibName:@"RerollingAspect"])) {
		/* rerolling individual dice */
		[self setWillRerollIndividualDiceUnderValue:NO];	//AM_willRerollIndividualDiceUnderValue = NO;
		[self setWillRerollIndividualDiceOverValue:NO];	//AM_willRerollIndividualDiceOverValue = NO;
		
		[self setRerollIndividualDiceUnderValue:0];	//AM_rerollIndividualDiceUnderValue = 0;
		[self setRerollIndividualDiceOverValue:0];	//AM_rerollIndividualDiceOverValue = 0;
		
		[self setRerollIndividualDicePrecedence:AMRerollBeforeModifiers];	//AM_rerollIndividualDicePrecedence = AMRerollBeforeModifiers;
		[self setRerollingIndividualDiceOptionsEnabled:NO];	//AM_rerollingIndividualDiceOptionsEnabled = NO;
		
		/* rerolling percentile dice */
		[self setWillRerollPercentileDiceUnderValue:NO];	//AM_willRollPercentileUnderValue = NO;
		[self setWillRerollPercentileDiceOverValue:NO];	//AM_willRollAsPercentileOverValue = NO;
		
		[self setRerollPercentileDiceUnderValue:0];	//AM_rerollPercentileDiceUnderValue = 0;
		[self setRerollPercentileDiceOverValue:0];	//AM_rerollPercentileDiceOverValue = 0;
		
		[self setRerollPercentileDicePrecedence:AMRerollBeforeModifiers];	//AM_rerollPercentileDicePrecedence = AMRerollBeforeModifiers;
		
		[self setWillRollAsPercentile:NO];	//AM_willRollAsPercentile = NO;
		[self setRerollingPercentileDiceOptionsEnabled:NO];	//AM_rerollingPercentileDiceOptionsEnabled = NO;

		[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(percentileRollApplyChanges:) name:@"AMModifyPercentileRoll" object:nil];
	}
	return self;
}

- (void)dealloc {
	[[NSNotificationCenter defaultCenter] removeObserver:self name:@"AMModifyPercentileRoll" object:nil];
	[super dealloc];
}

- (void)awakeFromNib {
	[self updateRerollingIndividualDiceOptionsEnabled];
	[self updateRerollingPercentileDiceOptionsEnabled];
}

- (NSString *)label {
	return @"Rerolling";
}

- (NSString *)identifier {
	return @"Rerolling";
}

- (void)percentileRollApplyChanges:(NSNotification *)aNotification {
	BOOL percentile = [[aNotification object] willRollAsPercentile];
	[self setWillRollAsPercentile:percentile];
	
	if (percentile) {
		[self setWillRerollIndividualDiceUnderValue:NO];
		[self setWillRerollIndividualDiceOverValue:NO];
		
		//[self setRerollIndividualDiceUnderValue:0];
		//[self setRerollIndividualDiceOverValue:0];
	}
}

- (void)loadValuesFromDefaults {
	[super loadValuesFromDefaults];
	
	id userDefaults = [NSUserDefaults standardUserDefaults];
	
	[self setWillRerollIndividualDiceUnderValue:[userDefaults boolForKey:AMWillRerollIndividualDiceUnderValueKey]];
	[self setWillRerollIndividualDiceOverValue:[userDefaults boolForKey:AMWillRerollIndividualDiceOverValueKey]];
	[self setRerollIndividualDiceUnderValue:[userDefaults integerForKey:AMRerollIndividualDiceUnderValueKey]];
	[self setRerollIndividualDiceOverValue:[userDefaults integerForKey:AMRerollIndividualDiceOverValueKey]];
	[self setRerollIndividualDicePrecedence:[userDefaults integerForKey:AMRerollIndividualDicePrecedenceKey]];

	[self setWillRerollPercentileDiceUnderValue:[userDefaults boolForKey:AMWillRerollPercentileDiceUnderValueKey]];
	[self setWillRerollPercentileDiceOverValue:[userDefaults boolForKey:AMWillRerollPercentileDiceOverValueKey]];
	[self setRerollPercentileDiceUnderValue:[userDefaults integerForKey:AMRerollPercentileDiceUnderValueKey]];
	[self setRerollPercentileDiceOverValue:[userDefaults integerForKey:AMRerollPercentileDiceOverValueKey]];
	[self setRerollPercentileDicePrecedence:[userDefaults integerForKey:AMRerollPercentileDicePrecedenceKey]];

	//[self setWillRollAsPercentile:[userDefaults boolForKey:AMWillRollAsPercentileKey]];
}

- (void)saveValuesToDefaults {
	[super saveValuesToDefaults];
	
	id userDefaults = [NSUserDefaults standardUserDefaults];
	
	[userDefaults setBool:[self willRerollIndividualDiceUnderValue] forKey:AMWillRerollIndividualDiceUnderValueKey];
	[userDefaults setBool:[self willRerollIndividualDiceOverValue] forKey:AMWillRerollIndividualDiceOverValueKey];
	[userDefaults setInteger:[self rerollIndividualDiceUnderValue] forKey:AMRerollIndividualDiceUnderValueKey];
	[userDefaults setInteger:[self rerollIndividualDiceOverValue] forKey:AMRerollIndividualDiceOverValueKey];
	[userDefaults setInteger:[self rerollIndividualDicePrecedence] forKey:AMRerollIndividualDicePrecedenceKey];

	[userDefaults setBool:[self willRerollPercentileDiceUnderValue] forKey:AMWillRerollPercentileDiceUnderValueKey];
	[userDefaults setBool:[self willRerollPercentileDiceOverValue] forKey:AMWillRerollPercentileDiceOverValueKey];
	[userDefaults setInteger:[self rerollPercentileDiceUnderValue] forKey:AMRerollPercentileDiceUnderValueKey];
	[userDefaults setInteger:[self rerollPercentileDiceOverValue] forKey:AMRerollPercentileDiceOverValueKey];
	[userDefaults setInteger:[self rerollPercentileDicePrecedence] forKey:AMRerollPercentileDicePrecedenceKey];

	//[userDefaults setBool:[self willRollAsPercentile] forKey:AMWillRollAsPercentileKey];
}

- (NSString *)summary {
	NSMutableArray *sentences = [NSMutableArray array];	//WithObject:[super summary];
	NSString *sentence;
	
	AMRerollPrecedence precedence;
	NSString *precedenceString;

	if ([self willRollAsPercentile]) {

		if ([self willRerollPercentileDiceUnderValue]) {
			sentence = [[NSString alloc] initWithFormat:@"A total result under %i will be rerolled.", [self rerollPercentileDiceUnderValue]];
			[sentences addObject:sentence];
			[sentence release];
		}

		if ([self willRerollPercentileDiceOverValue]) {
			sentence = [[NSString alloc] initWithFormat:@"A total result over %i will be rerolled.", [self rerollPercentileDiceOverValue]];
			[sentences addObject:sentence];
			[sentence release];
		}
		
		if ([self rerollingPercentileDiceOptionsEnabled]) {
			precedence = [self rerollPercentileDicePrecedence];
			if (precedence == AMRerollBeforeModifiers) {
				precedenceString = @"before";
			} else if (precedence == AMRerollAfterModifiers) {
				precedenceString = @"after";
			} else {
				precedenceString = @"neither before nor after";	// sanity-check
			}
			
			sentence = [[NSString alloc] initWithFormat:@"Rerolling of percentile dice will occur %@ the application of modifiers.", precedenceString];
			[sentences addObject:sentence];
			[sentence release];
		}

	} else {
		
		if ([self willRerollIndividualDiceUnderValue]) {
			sentence = [[NSString alloc] initWithFormat:@"Individual dice under %i will be rerolled.", [self rerollIndividualDiceUnderValue]];
			[sentences addObject:sentence];
			[sentence release];
		}
		
		if ([self willRerollIndividualDiceOverValue]) {
			sentence = [[NSString alloc] initWithFormat:@"Individual dice over %i will be rerolled.", [self rerollIndividualDiceOverValue]];
			[sentences addObject:sentence];
			[sentence release];
		}
		
		if ([self rerollingIndividualDiceOptionsEnabled]) {
			precedence = [self rerollIndividualDicePrecedence];

			if (precedence == AMRerollBeforeModifiers) {
				precedenceString = @"before";
			} else if (precedence == AMRerollAfterModifiers) {
				precedenceString = @"after";
			} else {
				precedenceString = @"neither before nor after";	// sanity-check
			}
			
			sentence = [[NSString alloc] initWithFormat:@"Rerolling of individual dice will occur %@ the application of modifiers.", precedenceString];
			[sentences addObject:sentence];
			[sentence release];
		}
	}

	return [sentences componentsJoinedByString:AMDoubleSpace];
}

@end

@implementation DXAspectControllerRerolling (RerollingIndividualDice)

- (BOOL)willRerollIndividualDiceUnderValue {
	return AM_willRerollIndividualDiceUnderValue;
}

- (void)setWillRerollIndividualDiceUnderValue:(BOOL)flag {
	AM_willRerollIndividualDiceUnderValue = flag;
	[[NSNotificationCenter defaultCenter] postNotificationName:@"AMModifiedSettings" object:self];
	[self updateRerollingIndividualDiceOptionsEnabled];
}

- (BOOL)willRerollIndividualDiceOverValue {
	return AM_willRerollIndividualDiceOverValue;
}

- (void)setWillRerollIndividualDiceOverValue:(BOOL)flag {
	AM_willRerollIndividualDiceOverValue = flag;
	[[NSNotificationCenter defaultCenter] postNotificationName:@"AMModifiedSettings" object:self];
	[self updateRerollingIndividualDiceOptionsEnabled];
}

- (int)rerollIndividualDiceUnderValue {
	return AM_rerollIndividualDiceUnderValue;
}

- (void)setRerollIndividualDiceUnderValue:(int)anInt {
	AM_rerollIndividualDiceUnderValue = anInt;
	[[NSNotificationCenter defaultCenter] postNotificationName:@"AMModifiedSettings" object:self];
}

- (int)rerollIndividualDiceOverValue {
	return AM_rerollIndividualDiceOverValue;
}

- (void)setRerollIndividualDiceOverValue:(int)anInt {
	AM_rerollIndividualDiceOverValue = anInt;
	[[NSNotificationCenter defaultCenter] postNotificationName:@"AMModifiedSettings" object:self];
}

- (AMRerollPrecedence)rerollIndividualDicePrecedence {
	return AM_rerollIndividualDicePrecedence;
}

- (void)setRerollIndividualDicePrecedence:(AMRerollPrecedence)aCondition {
	AM_rerollIndividualDicePrecedence = aCondition;
	[[NSNotificationCenter defaultCenter] postNotificationName:@"AMModifiedSettings" object:self];
}

/* used for enabling of precedence radio buttons */
- (BOOL)rerollingIndividualDiceOptionsEnabled {
	return AM_rerollingIndividualDiceOptionsEnabled;
}

- (void)setRerollingIndividualDiceOptionsEnabled:(BOOL)flag {
	AM_rerollingIndividualDiceOptionsEnabled = flag;
}

- (void)updateRerollingIndividualDiceOptionsEnabled {
	BOOL flag = (([self willRerollIndividualDiceUnderValue] || [self willRerollIndividualDiceOverValue]) ? YES : NO);
	[self setRerollingIndividualDiceOptionsEnabled:flag];
}

@end

@implementation DXAspectControllerRerolling (PercentileRerolling)

- (BOOL)willRerollPercentileDiceUnderValue {
	return AM_willRollPercentileUnderValue;
}

- (void)setWillRerollPercentileDiceUnderValue:(BOOL)flag {
	AM_willRollPercentileUnderValue = flag;
	[[NSNotificationCenter defaultCenter] postNotificationName:@"AMModifiedSettings" object:self];
	[self updateRerollingPercentileDiceOptionsEnabled];
}

- (BOOL)willRerollPercentileDiceOverValue {
	return AM_willRollAsPercentileOverValue;
}

- (void)setWillRerollPercentileDiceOverValue:(BOOL)flag {
	AM_willRollAsPercentileOverValue = flag;
	[[NSNotificationCenter defaultCenter] postNotificationName:@"AMModifiedSettings" object:self];
	[self updateRerollingPercentileDiceOptionsEnabled];
}

- (int)rerollPercentileDiceUnderValue {
	return AM_rerollPercentileDiceUnderValue;
}

- (void)setRerollPercentileDiceUnderValue:(int)anInt {
	AM_rerollPercentileDiceUnderValue = anInt;
	[[NSNotificationCenter defaultCenter] postNotificationName:@"AMModifiedSettings" object:self];
}

- (int)rerollPercentileDiceOverValue {
	return AM_rerollPercentileDiceOverValue;
}

- (void)setRerollPercentileDiceOverValue:(int)anInt {
	AM_rerollPercentileDiceOverValue = anInt;
	[[NSNotificationCenter defaultCenter] postNotificationName:@"AMModifiedSettings" object:self];
}

- (AMRerollPrecedence)rerollPercentileDicePrecedence {
	return AM_rerollPercentileDicePrecedence;
}

- (void)setRerollPercentileDicePrecedence:(AMRerollPrecedence)aCondition {
	AM_rerollPercentileDicePrecedence = aCondition;
	[[NSNotificationCenter defaultCenter] postNotificationName:@"AMModifiedSettings" object:self];
}

/* used for enabling of precedence radio buttons */
- (BOOL)rerollingPercentileDiceOptionsEnabled {
	return AM_rerollingPercentileDiceOptionsEnabled;
}

- (void)setRerollingPercentileDiceOptionsEnabled:(BOOL)flag {
	AM_rerollingPercentileDiceOptionsEnabled = flag;
}

- (void)updateRerollingPercentileDiceOptionsEnabled {
	BOOL flag = (([self willRerollPercentileDiceUnderValue] || [self willRerollPercentileDiceOverValue]) ? YES : NO);
	[self setRerollingPercentileDiceOptionsEnabled:flag];
}

@end

@implementation DXAspectControllerRerolling (PercentileDiceExtensions)

- (BOOL)willRollAsPercentile {
	return AM_willRollAsPercentile;
}

- (void)setWillRollAsPercentile:(BOOL)flag {
	AM_willRollAsPercentile = flag;
}

@end
