//
//  DXDieGroup.m
//  DiceX
//
//  Created by Andrew Merenbach on 25/11/2004.
//  Copyright 2004 Andrew Merenbach. All rights reserved.
//

#import "DXDieGroup.h"

#import "DXDie.h"

@implementation DXDieGroup

+ (void)initialize {
    srandom(time(NULL));
}

- (id)init {
	if (self = [super init]) {
		AM_total = 0;
		
		AM_dieCount = 0;
		AM_sidesPerDie = 0;
		
		AM_perDieBonusModifier = 0;
		AM_perDiePenaltyModifier = 0;
		AM_perDieMultiplierModifier = 1;
		
		AM_totalBonusModifier = 0;
		AM_totalPenaltyModifier = 0;
		AM_totalMultiplierModifier = 1;
				
		diceArray = [[NSMutableArray allocWithZone:[self zone]] init];
		
		AM_percentileRoll = NO;
	}
	return self;
}

- (void)dealloc {
	[diceArray release];
	diceArray = nil;
	
	[super dealloc];
}

/* cover method for log window controller, which needs to enumerator */
- (NSEnumerator *)dieEnumerator {
	return [diceArray objectEnumerator];
}

- (void)rollDice {
	[diceArray setArray:[self blankDice]];
	[diceArray makeObjectsPerformSelector:@selector(rollDie)];
}

- (void)rollDiceForPercentile {
	[diceArray setArray:[self blankDice]];
	[diceArray makeObjectsPerformSelector:@selector(rollDieForPercentile)];
}

- (void)sortDice {
	[diceArray sortUsingSelector:@selector(compare:)];
}

- (void)rerollPercentileDiceWithValuesUnder:(int)testValue withMods:(BOOL)mods {
	NSEnumerator *d = [diceArray objectEnumerator];
	id die;
	
	int dieValue;
	
	while (die = [d nextObject]) {
		dieValue = (mods ? [die total] : [die rollValue]);
		while (dieValue < testValue) {
			[die rollDie];
			dieValue = (mods ? [die total] : [die rollValue]);
		}
	}
}

- (void)rerollPercentileDiceWithValuesOver:(int)testValue withMods:(BOOL)mods {
	NSEnumerator *d = [diceArray objectEnumerator];
	id die;
	
	int dieValue;
	
	while (die = [d nextObject]) {
		dieValue = (mods ? [die total] : [die rollValue]);
		while (dieValue > testValue) {
			[die rollDie];
			dieValue = (mods ? [die total] : [die rollValue]);
		}
	}
}

- (void)tallyDice {
	int total = 0;

	NSEnumerator *d = [diceArray objectEnumerator];
	id die;
	
	unsigned int possibleMax = [self dieCount] * [self sidesPerDie];
	double percentage;
	unsigned int dieTotal;
	
	while (die = [d nextObject]) {
		if ([die isIncluded]) {
			[die setDieBonusModifier:[self perDieBonusModifier]];
			[die setDiePenaltyModifier:[self perDiePenaltyModifier]];
			[die setDieMultiplierModifier:[self perDieMultiplierModifier]];
			
			[die recalculateTotal];
			dieTotal = [die total];

			total += dieTotal;
			
			percentage = (double)dieTotal / (double)possibleMax * (double)100;
			[die setDiePercentage:percentage];
		}
	}
	
	total += [self totalBonusModifier];
	total -= [self totalPenaltyModifier];
	total *= [self totalMultiplierModifier];
	
	[self setTotal:total];
	
	percentage = (double)total / (double)possibleMax * (double)100;
	[self setPercentage:percentage];
}

- (void)tallyDiceForPercentileRoll:(BOOL)willPercentileZeroEqualMax {
	int total = 0;

	NSEnumerator *d = [diceArray reverseObjectEnumerator];	// reverse so that we can start with the "ones" place-value column
	id die;
	
	unsigned int placeValueColumn = (-1);
	unsigned int placeValue;
	
	while (die = [d nextObject]) {
		if ([die isIncluded]) {
			//[die setDieBonusModifier:[self perDieBonusModifier]];
			//[die setDiePenaltyModifier:[self perDiePenaltyModifier]];
			
			[die recalculateTotal];
			
			placeValue = (unsigned int)(pow((double)10, (double)(++placeValueColumn)));
			total += (placeValue * [die total]);

			[die setDiePercentage:0];
		}
	}

	if (total == 0 && willPercentileZeroEqualMax) {
		total = 100;
	}
	
	total += [self totalBonusModifier];
	total -= [self totalPenaltyModifier];
	total *= [self totalMultiplierModifier];
		
	[self setTotal:total];


	double possibleMax = pow((double)10, (double)[self dieCount]);
	if (!willPercentileZeroEqualMax) {
		--possibleMax;
	}
	double p = (double)total / (double)possibleMax * (double)100;
	[self setPercentage:p];
}

// ------- semi-old stuff below

/* return an autoreleased, mutable array
	of dice with default attributes */
- (NSMutableArray *)blankDice {
	unsigned n;
	unsigned dieCount = [self dieCount];
	unsigned sidesPerDie = [self sidesPerDie];
	DXDie *die;
	
	NSMutableArray *array = [NSMutableArray arrayWithCapacity:dieCount];
	
	for (n = 0; n < dieCount; n++) {
		die = [[DXDie alloc] init];
		[die setNumberOfSides:sidesPerDie];
		[die setDieBonusModifier:[self perDieBonusModifier]];
		[die setDiePenaltyModifier:[self perDiePenaltyModifier]];
		[array addObject:die];
		[die release];
	}
	
	return array;
}

- (void)dropLowRollsWithCount:(unsigned int)rollDrops {
	unsigned countOfDice = [diceArray count];
	if (rollDrops > 0 && countOfDice > 0) {	// [dice count] is a sanity-check
		NSArray *rollsSubarray = [diceArray subarrayWithRange:NSMakeRange(0, rollDrops)];
		[rollsSubarray makeObjectsPerformSelector:@selector(drop)];
	}
}

- (void)dropHighRollsWithCount:(unsigned int)rollDrops {
	unsigned countOfDice = [diceArray count];
	if (rollDrops > 0 && countOfDice > 0) {	// [dice count] is a sanity-check
		NSArray *rollsSubarray = [diceArray subarrayWithRange:NSMakeRange(countOfDice - rollDrops, rollDrops)];
		[rollsSubarray makeObjectsPerformSelector:@selector(drop)];
	}
}

@end


@implementation DXDieGroup (Accessors)

- (int)total {
	return AM_total;
}

- (void)setTotal:(int)anInt {
	AM_total = anInt;
}

/* universal dice property accessors */
- (unsigned)dieCount {
	return AM_dieCount;
}

- (void)setDieCount:(unsigned)aNumber {
	AM_dieCount = aNumber;
}

- (unsigned)sidesPerDie {
	return AM_sidesPerDie;
}

- (void)setSidesPerDie:(unsigned)aNumber {
	AM_sidesPerDie = aNumber;
}

- (unsigned int)perDieBonusModifier {
	return AM_perDieBonusModifier;
}

- (void)setPerDieBonusModifier:(unsigned int)anInt {
	AM_perDieBonusModifier = anInt;
}

- (unsigned int)perDiePenaltyModifier {
	return AM_perDiePenaltyModifier;
}

- (void)setPerDiePenaltyModifier:(unsigned int)anInt {
	AM_perDiePenaltyModifier = anInt;
}

- (unsigned int)perDieMultiplierModifier {
	return AM_perDieMultiplierModifier;
}

- (void)setPerDieMultiplierModifier:(unsigned int)anInt {
	AM_perDieMultiplierModifier = anInt;
}

- (unsigned int)totalBonusModifier {
	return AM_totalBonusModifier;
}

- (void)setTotalBonusModifier:(unsigned int)anInt {
	AM_totalBonusModifier = anInt;
}

- (unsigned int)totalPenaltyModifier {
	return AM_totalPenaltyModifier;
}

- (void)setTotalPenaltyModifier:(unsigned int)anInt {
	AM_totalPenaltyModifier = anInt;
}

- (unsigned int)totalMultiplierModifier {
	return AM_totalMultiplierModifier;
}

- (void)setTotalMultiplierModifier:(unsigned int)anInt {
	AM_totalMultiplierModifier = anInt;
}

- (double)percentage {
	return AM_percentage;
}

- (void)setPercentage:(double)aDouble {
	AM_percentage = aDouble;
}

- (BOOL)isPercentileRoll {
	return AM_percentileRoll;
}

- (void)setIsPercentileRoll:(BOOL)flag {
	AM_percentileRoll = flag;
}

@end

