//
//  DXLogWindowController+EntryManagement.m
//  DiceX
//
//  Created by Andrew Merenbach on 8/1/07.
//  Copyright 2007 __MyCompanyName__. All rights reserved.
//

#import "DXLogWindowController+EntryManagement.h"


@implementation DXLogWindowController (ExpandAndCollapseAndScroll)

- (BOOL)canCollapseOrExpandLogEntries {
	return (![self isProcessingRolls] && ([self fetchedBaseNodesCount] > 0));
}

- (void)expandMostRecentEntry:(id)sender {
	NSSortDescriptor *sd = [[self logSortDescriptors] objectAtIndex:0];
	if ([sd ascending]) {
		[self expandLastEntry:sender];
	} else {
		[self expandFirstEntry:sender];
	}
}

- (void)scrollMostRecentEntryToVisible:(id)sender {
	NSSortDescriptor *sd = [[self logSortDescriptors] objectAtIndex:0];
	if ([sd ascending]) {
		[logOutlineView scrollLastRowToVisible:nil];
	} else {
		[logOutlineView scrollFirstRowToVisible:nil];
	}
}

/* cover method for log outline view */
- (void)expandFirstEntry:(id)sender {
	[logOutlineView expandFirstItem:sender];
}

/* cover method for log outline view */
- (void)expandLastEntry:(id)sender {
	[logOutlineView expandLastItem:sender];
}

- (void)collapseAllEntries:(id)sender {
	if (![self isProcessingRolls]) {
		[self updateFetchedBaseNodesCount];	// [kludge] necessary to refresh for first row to expand
		//[logTreeController fetch:sender];
		[logOutlineView collapseAllRows:sender];
		//[NSThread detachNewThreadSelector:@selector(collapseAllEntriesThread:) toTarget:self withObject:sender];
	} else {
		[self displayIsBusyAlert];
	}
}

/*- (void)collapseAllEntriesThread:(id)sender {
	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];

	if (![self isBusyWithLogEntries]) {	// sanity-check
		[self setBusyWithLogEntries:YES];
	
		[self updateFetchedBaseNodesCount];	// [kludge] necessary to refresh for first row to expand
		//[logTreeController fetch:sender];
		[logOutlineView collapseAllRows:sender];
		
		[self setBusyWithLogEntries:NO];
	}
	
	[pool release];
}*/

- (void)expandAllEntries:(id)sender {
	if (![self isProcessingRolls]) {
		[self updateFetchedBaseNodesCount];	// [kludge] necessary to refresh for first row to expand
		//[logTreeController fetch:sender];
		[logOutlineView expandAllRows:sender];
		//[NSThread detachNewThreadSelector:@selector(collapseAllEntriesThread:) toTarget:self withObject:sender];
	} else {
		[self displayIsBusyAlert];
	}
}

/*- (void)expandAllEntriesThread:(id)sender {
	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];

	if (![self isBusyWithLogEntries]) {	// sanity-check
		[self setBusyWithLogEntries:YES];
	
		[self updateFetchedBaseNodesCount];	// [kludge] necessary to refresh for first row to expand
		//[logTreeController fetch:sender];
		[logOutlineView expandAllRows:sender];
		
		[self setBusyWithLogEntries:NO];
	}
	
	[pool release];
}*/

/* transient (not Core Data-style "transient"; just transient) properties */
- (BOOL)autoscrollEntries {
	return AM_autoscrollEntries;
}

- (void)setAutoscrollEntries:(BOOL)flag {
	AM_autoscrollEntries = flag;
}

- (BOOL)autoExpandEntries {
	return AM_autoExpandEntries;
}

- (void)setAutoExpandEntries:(BOOL)flag {
	AM_autoExpandEntries = flag;
}

@end

@implementation DXLogWindowController (EntryRemoval)

- (BOOL)canRemoveLogEntry {
	return (![self isProcessingRolls]
			&& [logTreeController canRemove]
			&& ![[logTreeController selection] valueForKey:@"parent"]
			&& [[self window] isVisible]);
}

- (BOOL)canClearLogEntries {
	return (![self isProcessingRolls] && [self fetchedBaseNodesCount] > 0 && [[self window] isVisible]);
}

- (IBAction)clearLog:(id)sender {
	if (![self isProcessingRolls]) {
		if ([self fetchedBaseNodesCount] > 0) {
			NSAlert *alert = [NSAlert alertWithMessageText:@"Warning!"
					defaultButton:@"OK"
					alternateButton:@"Cancel"
					otherButton:nil
					informativeTextWithFormat:@"Are you sure that you want to clear the roll log?"];
			[alert setAlertStyle:NSWarningAlertStyle];	// NSCriticalAlertStyle
			[alert beginSheetModalForWindow:[self window] modalDelegate:self
				didEndSelector:@selector(clearSheetDidEnd:returnCode:) contextInfo:NULL];	// contextInfo:
		}
	} else {
		[self displayIsBusyAlert];
	}
}

- (void)clearSheetDidEnd:(NSWindow *)sheet returnCode:(int)returnCode {	// contextInfo:(void *)contextInfo
	if (returnCode == NSOKButton) {
		NSMutableArray *ips = [[NSMutableArray alloc] init];
		
		unsigned count = [[logTreeController arrangedObjects] count];
		NSIndexPath *ip;

		unsigned idx;
		for (idx = 0; idx < count; ++idx) {
			ip = [[NSIndexPath alloc] initWithIndex:idx];
			[ips addObject:ip];
			[ip release];
		}
		
		[logTreeController removeObjectsAtArrangedObjectIndexPaths:ips];
		
		[ips release];
		
		[self updateFetchedBaseNodesCount];
		//[NSThread detachNewThreadSelector:@selector(clearAllRollsThread) toTarget:self withObject:nil];
	}
}

/*- (void)clearAllRollsThread {
	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
	
	if (![self isBusyWithLogEntries]) {	// sanity-check
		
		[self setBusyWithLogEntries:YES];
		

			NSMutableArray *ips = [[NSMutableArray alloc] init];
			
			unsigned count = [[logTreeController arrangedObjects] count];
			NSIndexPath *ip;

			unsigned idx;
			for (idx = 0; idx < count; ++idx) {
				ip = [[NSIndexPath alloc] initWithIndex:idx];
				[ips addObject:ip];
				[ip release];
			}
			
			[logTreeController removeObjectsAtArrangedObjectIndexPaths:ips];
			
			[ips release];
			
			[self updateFetchedBaseNodesCount];
			
		[self setBusyWithLogEntries:NO];
	}
		
	[pool release];
}*/

- (void)removeSelectedEntries:(id)sender {
	if (![self isProcessingRolls]) {
		NSArray *indexPaths = [logTreeController selectionIndexPaths];
		[logTreeController removeObjectsAtArrangedObjectIndexPaths:indexPaths];
		[self updateFetchedBaseNodesCount];
		//[NSThread detachNewThreadSelector:@selector(removeSelectedEntries:) toTarget:self withObject:sender];
	} else {
		[self displayIsBusyAlert];
	}
}

/*- (void)removeSelectedEntriesThread:(id)sender {
	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];

	if (![self isBusyWithLogEntries]) {	// sanity-check
		
		[self setBusyWithLogEntries:YES];
		
		NSArray *indexPaths = [logTreeController selectionIndexPaths];
		[logTreeController removeObjectsAtArrangedObjectIndexPaths:indexPaths];
		[self updateFetchedBaseNodesCount];
		
		[self setBusyWithLogEntries:NO];
	}
	
	[pool release];
}*/

@end

@implementation DXLogWindowController (OutlineViewDataSource)

- (void)delete:(id)sender {
	if ([self canRemoveLogEntry]) {
		[self removeSelectedEntries:sender];
	}
}

- (id)outlineView:(NSOutlineView *)outlineView child:(int)index ofItem:(id)item {
	return nil;
}

- (BOOL)outlineView:(NSOutlineView *)outlineView isItemExpandable:(id)item {
	return NO;
}

- (int)outlineView:(NSOutlineView *)outlineView numberOfChildrenOfItem:(id)item {
	return 0;
}

- (id)outlineView:(NSOutlineView *)outlineView objectValueForTableColumn:(NSTableColumn *)tableColumn byItem:(id)item {
	return nil;
}

@end

