//
//  DXLogWindowController.m
//  DiceX
//
//  Created by Andrew Merenbach on 22/11/2004.
//  Copyright 2004 Andrew Merenbach. All rights reserved.
//

#import "DXLogWindowController.h"
#import "DXLogWindowController+EntryManagement.h"
#import "DXToolbarFactory.h"

#import "DXDieNode.h"

#import "DXDieGroup.h"
#import "DXDie.h"


@implementation DXLogWindowController

- (id)init {
	if (self = [super initWithWindowNibName:@"Log"]) {
		NSZone *thisZone = [self zone];
		
		toolbarItems = [[NSMutableDictionary allocWithZone:thisZone] init];
		
		{
			NSSortDescriptor *logSortDescriptor = [[NSSortDescriptor allocWithZone:thisZone] initWithKey:@"orderHint" ascending:YES];
			[self willChangeValueForKey:@"logSortDescriptors"];
			AM_logSortDescriptors = [[NSArray allocWithZone:thisZone] initWithObjects:logSortDescriptor, /*dropSortDescriptor,*/ nil];
			[self didChangeValueForKey:@"logSortDescriptors"];
			[logSortDescriptor release];
		}
		
		AM_fetchedBaseNodesCount = 0;

		//[self setCountOfNewNodes:0];
		//[dropSortDescriptor release];
		
		[self setAutoscrollEntries:YES];
		[self setAutoExpandEntries:NO];
		
		[self setLastMasterOrderHint:0];
		
		[self setProcessingRolls:NO];
		
		[self setSaving:NO];
		
		[self setPersistentStoreCoordinatorRef:nil];
		
		//[self setBusyWithLogEntries:NO];
		//logManagedObjectContext = [[NSManagedObjectContext allocWithZone:thisZone] init];
		
		//[logManagedObjectContext setManagedObjectModel:[[NSApp delegate] managedObjectModel]];
	}
	return self;
}

- (void)dealloc {
	[self setPersistentStoreCoordinatorRef:nil];	// reset weak link

	[toolbarItems release];
	toolbarItems = nil;
	
	[AM_logSortDescriptors release];
	AM_logSortDescriptors = nil;
	
	//[self setSaving:NO];
	
	//[logManagedObjectContext release];
	//logManagedObjectContext = nil;
	
	//[AM_fetchedBaseNodes release];
	//AM_fetchedBaseNodes = nil;
	
	[super dealloc];
}

- (void)windowDidLoad {
	//psCoordinator = [[[NSApp delegate] managedObjectContext] persistentStoreCoordinator];
	//[logManagedObjectContext setPersistentStoreCoordinator:psCoordinator];
	
	[self updateFetchedBaseNodesCount];
	//[self setupToolbar:[self window]];
	[toolbarFactory setupToolbarWithIdentifier:@"LogToolbar"];

	[logOutlineView setTarget:self];
	[logOutlineView setDoubleAction:@selector(doubleClickedEntry:)];
}

- (BOOL)validateMenuItem:(NSMenuItem *)menuItem {
	BOOL b = YES;
	
	if ([menuItem action] == @selector(clearLog:)) {
		b = [self canClearLogEntries];
	} else if ([menuItem action] == @selector(delete:)) {
		b = [self canRemoveLogEntry];
	} else if ([menuItem action] == @selector(removeSelectedEntries:)) {
		b = [self canRemoveLogEntry];
	} else if ([menuItem action] == @selector(exportLog:)) {
		b = [self canExportLogEntries];
	} else if ([menuItem action] == @selector(collapseAllEntries:)) {
		b = [self canCollapseOrExpandLogEntries];
	} else if ([menuItem action] == @selector(expandAllEntries:)) {
		b = [self canCollapseOrExpandLogEntries];
	}

	return b;
}

- (BOOL)validateToolbarItem:(NSToolbarItem *)theItem
{
	BOOL b = YES;

	if ([[theItem itemIdentifier] isEqualToString:@"ClearAll"]) {
		b = [self canClearLogEntries];
	} else if ([[theItem itemIdentifier] isEqualToString:@"RemoveRoll"]) {
		b = [self canRemoveLogEntry];
	} else if ([[theItem itemIdentifier] isEqualToString:@"Export"]) {
		b = [self canExportLogEntries];
	} else if ([[theItem itemIdentifier] isEqualToString:@"CollapseAll"]) {
		b = [self canCollapseOrExpandLogEntries];
	} else if ([[theItem itemIdentifier] isEqualToString:@"ExpandAll"]) {
		b = [self canCollapseOrExpandLogEntries];
	}
	
	return b;
}

- (void)recalculateMasterOrderHints {
	int orderHint = 0;

	NSArray *sortedBaseNodes = [self fetchBaseNodes];

	NSEnumerator *e;
	{
		NSSortDescriptor *sd = [[self logSortDescriptors] objectAtIndex:0];
		if ([sd ascending]) {
			e = [sortedBaseNodes objectEnumerator];	
		} else {
			e = [sortedBaseNodes reverseObjectEnumerator];
		}
	}
	
	id node;
	while (node = [e nextObject]) {
		NSNumber *newOrderHint = [NSNumber numberWithInt:(orderHint++)];
		[node setValue:newOrderHint forKey:@"orderHint"];
	}
	
	[self setLastMasterOrderHint:orderHint];
}

/* [TODO][BUG] sort out race condition? */
- (void)logRolls:(NSArray *)rolls {
	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];

	//NSManagedObjectContext *context = [[NSApp delegate] managedObjectContext];		
	NSManagedObjectContext *context = [[NSManagedObjectContext alloc] init];
	[context setPersistentStoreCoordinator:[self persistentStoreCoordinatorRef]];
	[[context undoManager] disableUndoRegistration];
	
	NSEntityDescription *dieNodeEntity = [NSEntityDescription entityForName:@"DieNode" inManagedObjectContext:context];
	DXDieNode *masterNode, *detailNode;
		
	[self performSelectorOnMainThread:@selector(recalculateMasterOrderHints) withObject:nil waitUntilDone:YES];
	int masterOrderHint = [self lastMasterOrderHint];
	int detailOrderHint;
	
	NSString *newRollValue;
	
	NSString *rollValueString;
	
	NSEnumerator *d = [rolls objectEnumerator];
	DXDieGroup *dieGroup;
	
	while ([self isProcessingRolls] && (dieGroup = [d nextObject])) {

		detailOrderHint = (-1);	
			
		if (masterNode = [[DXDieNode alloc] initWithEntity:dieNodeEntity insertIntoManagedObjectContext:context]) {
			[masterNode setNumberOfDicePerRoll:[NSNumber numberWithUnsignedInt:[dieGroup dieCount]]];
			[masterNode setNumberOfSidesPerDie:[NSNumber numberWithUnsignedInt:[dieGroup sidesPerDie]]];
			[masterNode setBonusModifier:[NSNumber numberWithInt:[dieGroup totalBonusModifier]]];
			[masterNode setPenaltyModifier:[NSNumber numberWithInt:[dieGroup totalPenaltyModifier]]];
			[masterNode setMultiplierModifier:[NSNumber numberWithInt:[dieGroup totalMultiplierModifier]]];
			[masterNode setTotal:[NSNumber numberWithInt:[dieGroup total]]];
			[masterNode setPercentage:[NSNumber numberWithDouble:[dieGroup percentage]]];
			[masterNode setIsIncluded:[NSNumber numberWithBool:YES]];
			[masterNode setIsPercentileRoll:[NSNumber numberWithBool:[dieGroup isPercentileRoll]]];
			
			{
				newRollValue = [[NSString alloc] initWithFormat:@"%@d%@",
					[masterNode numberOfDicePerRoll], [masterNode numberOfSidesPerDie]];
				[masterNode setRollValue:newRollValue];
				[newRollValue release];
			}
			
			[masterNode setOrderHint:[NSNumber numberWithInt:(++masterOrderHint)]];
			
			NSEnumerator *e = [dieGroup dieEnumerator];
			DXDie *die;
			
			while (die = [e nextObject]) {
				if ([self isProcessingRolls] && (detailNode = [[DXDieNode alloc] initWithEntity:dieNodeEntity insertIntoManagedObjectContext:context])) {
					[detailNode setNumberOfDicePerRoll:[NSNumber numberWithUnsignedInt:1]];
					[detailNode setNumberOfSidesPerDie:[NSNumber numberWithUnsignedInt:[die numberOfSides]]];
					[detailNode setBonusModifier:[NSNumber numberWithInt:[die dieBonusModifier]]];
					[detailNode setPenaltyModifier:[NSNumber numberWithInt:[die diePenaltyModifier]]];
					[detailNode setMultiplierModifier:[NSNumber numberWithInt:[die dieMultiplierModifier]]];
					[detailNode setTotal:[NSNumber numberWithInt:[die total]]];
					[detailNode setPercentage:[NSNumber numberWithDouble:[die diePercentage]]];
					[detailNode setIsIncluded:[NSNumber numberWithBool:[die isIncluded]]];
					[detailNode setIsPercentileRoll:[NSNumber numberWithBool:[dieGroup isPercentileRoll]]];	// we take from the die group, not the individual die
					
					rollValueString = [[NSString alloc] initWithFormat:@"%i", [die rollValue]];
					[detailNode setRollValue:rollValueString];
					[rollValueString release];
					
					[detailNode setOrderHint:[NSNumber numberWithInt:(++detailOrderHint)]];
					
					[masterNode addChildrenObject:detailNode];
					[detailNode release];
				}
			}
			
			[masterNode release];
		}
	}
	[self setLastMasterOrderHint:masterOrderHint];
	
	[self setSaving:YES];
	NSError *error;
	if (![context save:&error]) {
		[NSApp performSelectorOnMainThread:@selector(presentError:) withObject:error waitUntilDone:YES];
	}
	[self setSaving:NO];

	//[logTreeController performSelectorOnMainThread:@selector(fetch:) withObject:nil waitUntilDone:YES];
	[self performSelectorOnMainThread:@selector(finishLoggingRoll:) withObject:rolls waitUntilDone:YES];
	
	//[self setProcessingRolls:NO];


	[context release];
	
	[pool release];
}

- (void)finishLoggingRoll:(NSArray *)rolls {
	[self updateFetchedBaseNodesCount];
	
	NSError *error = nil;
	if ([logTreeController fetchWithRequest:nil merge:YES error:&error]) {
		if ([self autoExpandEntries]) {
			int numberOfRows = [logOutlineView numberOfRows];
			unsigned count = [rolls count];
			
			if (numberOfRows >= count) {
				int startingRow = numberOfRows - count;
				id item;
				
				int i;
				for (i = numberOfRows - 1; i >= startingRow && [self isProcessingRolls]; --i) {
					item = [logOutlineView itemAtRow:i];
					[logOutlineView expandItem:item];
				}
			}
		}
		
		if ([self autoscrollEntries]) {
			[self scrollMostRecentEntryToVisible:nil];
		}
	} else {
		[[NSApplication sharedApplication] presentError:error];
	}
	
	[self setProcessingRolls:NO];
}

- (void)displayIsBusyAlert {
	NSAlert *alert = [NSAlert alertWithMessageText:@"Action Error"
			defaultButton:@"Cancel"
			alternateButton:nil
			otherButton:nil
			informativeTextWithFormat:@"The log is currently busy performing another action."];
	[alert setAlertStyle:NSInformationalAlertStyle];
	[alert beginSheetModalForWindow:[self window] modalDelegate:nil didEndSelector:NULL contextInfo:nil];
}

// expand or collapse a double-clicked row
- (void)doubleClickedEntry:(id)sender {
	int row = [logOutlineView selectedRow];
	if (row != -1) {
		id item = [logOutlineView itemAtRow:row];
		if ([logOutlineView isExpandable:item]) {
			if ([logOutlineView isItemExpanded:item]) {
				[logOutlineView collapseItem:item];
			} else {
				[logOutlineView expandItem:item];
			}
		}
	}
}

@end

@implementation DXLogWindowController (Export)

- (BOOL)canExportLogEntries {
	return (![self isProcessingRolls] && [self fetchedBaseNodesCount] > 0 && [[self window] isVisible]);
}

- (IBAction)exportLog:(id)sender {
	if (![self isProcessingRolls]) {
		NSSavePanel *sp;
		
		// create or get the shared instance of NSSavePanel
		sp = [NSSavePanel savePanel];

		// set up new attributes
		//[sp setAccessoryView:newView];
		[sp setCanSelectHiddenExtension:YES];
		//[sp setRequiredFileType:@"rtf"];
		[sp setRequiredFileType:@"html"];	// although we're technically XHTML, we don't want
											// our files to open in SubEthaEdit, GoLive, etc.
		
		// we can, in init method, do the following:
		//[[NSFileManager defaultManager] changeCurrentDirectoryPath:NSHomeDirectory()];
		// in our sheet method, we can for 'forDirectory:' argument, use the following:
		//[[NSFileManager defaultManager] currentDirectoryPath]
		
		[sp
			beginSheetForDirectory:nil
			file:@"DiceX Roll Log"
			modalForWindow:[self window]
			modalDelegate:self
			didEndSelector:@selector(exportPanelDidEnd:returnCode:contextInfo:)
			contextInfo:NULL];
	} else {
		[self displayIsBusyAlert];
	}
}

- (void)exportPanelDidEnd:(NSSavePanel *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo
{
	if (returnCode == NSOKButton) {
		NSString *path = [[NSBundle mainBundle] pathForResource:@"export-template" ofType:@"xhtml"];
		NSMutableString *xhtml = [[NSMutableString alloc] initWithContentsOfFile:path];
		NSString *seekString = @"<!-- body -->";
		
		if (nil != xhtml) {
			NSRange range = [xhtml rangeOfString:seekString];
			if (NSNotFound != range.location) {
				[xhtml replaceCharactersInRange:range withString:[self xhtmlLogRepresentation]];
			}
			
			[xhtml writeToFile:[sheet filename] atomically:YES];
		}
		
		/*[xhtml replaceOccurrencesOfString:seekString
			withString:[self xhtmlLogRepresentation]
			options:NSLiteralSearch
			range:NSMakeRange(0, [xhtml length])];*/

		
		/*
		NSXMLDocument *doc = [[NSXMLDocument alloc] initWithXMLString:xhtml
			options:NSXMLDocumentTidyXML
			error:NULL];
		
		NSAttributedString *str = [[NSAttributedString alloc] initWithHTML:[doc XMLData]
			documentAttributes:NULL];
	
		NSData *rtfData = [str RTFFromRange:NSMakeRange(0, [str length]) documentAttributes:nil];
		[rtfData writeToFile:[sheet filename] atomically:YES];
		
		[str release];
		[doc release];
		*/
		
		
		/*NSData *xhtmlData = [xhtml dataUsingEncoding:NSUTF8StringEncoding];
		
		NSMutableAttributedString *attributedString = [[NSAttributedString alloc] initWithHTML:xhtmlData documentAttributes:NULL];
		NSData *rtfData = [attributedString RTFFromRange:NSMakeRange(0, [attributedString length]) documentAttributes:nil];
		[rtfData writeToFile:[sheet filename] atomically:YES];
		*/
		
		//[NSThread detachNewThreadSelector:@selector(exportEntriesThreadWithFilename) toTarget:self withObject:[sheet filename]];
	}
}

- (NSString *)xhtmlLogRepresentation {
	NSMutableString *table = [NSMutableString string];
	[table appendString:@"<table id=\"page-header\" border=\"0\" cellspacing=\"0\" cellpadding=\"0\">\n"];
		[table appendString:@"<tr><td>\n"];
		[table appendString:@"Below is a log of your dice rolls, in chronological order, from the DiceX log.\n"];
		[table appendString:@"</td></tr>\n"];
	[table appendString:@"</table>\n"];
	
	NSMutableString *rollCaption = [[NSMutableString alloc] init];
	[rollCaption appendString:@"<tr class=\"roll-caption\">\n"];
		[rollCaption appendString:@"<td colspan=\"8\">Roll %i</td>\n"];
	[rollCaption appendString:@"</tr>\n"];
	
	NSMutableString *rollHeader = [[NSMutableString alloc] init];
	[rollHeader appendString:@"<tr class=\"roll-header\">\n"];
		[rollHeader appendString:@"<th>Roll</th>\n"];
		[rollHeader appendString:@"<th>Bonus</th>\n"];
		[rollHeader appendString:@"<th>Penalty</th>\n"];
		[rollHeader appendString:@"<th>Multiplier</th>\n"];
		[rollHeader appendString:@"<th>Total</th>\n"];
		[rollHeader appendString:@"<th>Percentage</th>\n"];
		[rollHeader appendString:@"<th>Dropped?</th>\n"];
		[rollHeader appendString:@"<th>Percentile?</th>\n"];
	[rollHeader appendString:@"</tr>\n"];
	
	NSMutableString *rollCellTemplate = [[NSMutableString alloc] init];
	[rollCellTemplate appendString:@"<td>\n"];
		[rollCellTemplate appendString:@"%@\n"];
	[rollCellTemplate appendString:@"</td>\n"];
	
	unsigned rollNumber = 1;
	
	{
		NSString *roll;
		NSString *bonus;
		NSString *penalty;
		NSString *multiplier;
		NSString *total;
		NSString *percentage;
		NSString *isDropped;
		NSString *isPercentile;

		NSEnumerator *e = [[self fetchBaseNodes] objectEnumerator];
		NSEnumerator *f;
		id proxyObj;
		id childObj;
		
		while (proxyObj = [e nextObject]) {
			[table appendString:@"<table class=\"roll-table\" border=\"0\" cellspacing=\"0\" cellpadding=\"0\">\n"];

			roll = [[NSString alloc] initWithFormat:@"%@\n", [proxyObj valueForKey:@"rollValue"]];
			bonus = [[NSString alloc] initWithFormat:@"%@\n", [proxyObj valueForKey:@"bonusModifier"]];
			penalty = [[NSString alloc] initWithFormat:@"%@\n", [proxyObj valueForKey:@"penaltyModifier"]];
			multiplier = [[NSString alloc] initWithFormat:@"%@\n", [proxyObj valueForKey:@"multiplierModifier"]];
			total = [[NSString alloc] initWithFormat:@"%@\n", [proxyObj valueForKey:@"total"]];
			percentage = [[NSString alloc] initWithFormat:@"%@\n", [proxyObj valueForKey:@"percentage"]];
			isDropped = [[NSString alloc] initWithString:@"-\n"];
			isPercentile = [[NSString alloc] initWithFormat:@"%@\n", ([[proxyObj valueForKey:@"isPercentileRoll"] boolValue] ? @"yes" : @"no")];
			
			[table appendFormat:rollCaption, (rollNumber++)];
			[table appendString:rollHeader];
			[table appendString:@"<tr class=\"roll-summary\">\n"];
				[table appendFormat:rollCellTemplate, roll];
				[table appendFormat:rollCellTemplate, bonus];
				[table appendFormat:rollCellTemplate, penalty];
				[table appendFormat:rollCellTemplate, multiplier];
				[table appendFormat:rollCellTemplate, total];
				[table appendFormat:rollCellTemplate, percentage];
				[table appendFormat:rollCellTemplate, isDropped];
				[table appendFormat:rollCellTemplate, isPercentile];
			[table appendString:@"</tr>\n"];
			
			[roll release];
			[bonus release];
			[penalty release];
			[multiplier release];
			[total release];
			[percentage release];
			[isDropped release];
			[isPercentile release];
			
			f = [[proxyObj valueForKey:@"children"] objectEnumerator];
			while (childObj = [f nextObject]) {
				roll = [[NSString alloc] initWithFormat:@"%@\n", [childObj valueForKey:@"rollValue"]];
				bonus = [[NSString alloc] initWithFormat:@"%@\n", [childObj valueForKey:@"bonusModifier"]];
				penalty = [[NSString alloc] initWithFormat:@"%@\n", [childObj valueForKey:@"penaltyModifier"]];
				multiplier = [[NSString alloc] initWithFormat:@"%@\n", [childObj valueForKey:@"multiplierModifier"]];
				total = [[NSString alloc] initWithFormat:@"%@\n", [childObj valueForKey:@"total"]];
				percentage = [[NSString alloc] initWithFormat:@"%@\n", [childObj valueForKey:@"percentage"]];
				isDropped = [[NSString alloc] initWithFormat:@"%@\n", ([[childObj valueForKey:@"isIncluded"] boolValue] ? @"no" : @"yes")];
				isPercentile = [[NSString alloc] initWithString:@"-\n"];
				
				[table appendString:@"<tr>\n"];
					[table appendFormat:rollCellTemplate, roll];
					[table appendFormat:rollCellTemplate, bonus];
					[table appendFormat:rollCellTemplate, penalty];
					[table appendFormat:rollCellTemplate, multiplier];
					[table appendFormat:rollCellTemplate, total];
					[table appendFormat:rollCellTemplate, percentage];
					[table appendFormat:rollCellTemplate, isDropped];
					[table appendFormat:rollCellTemplate, isPercentile];
				[table appendString:@"</tr>\n"];
				
				[roll release];
				[bonus release];
				[penalty release];
				[multiplier release];
				[total release];
				[percentage release];
				[isDropped release];
				[isPercentile release];
			}
			[table appendString:@"</table>\n"];
		}
	}		
	
	[rollCaption release];
	[rollHeader release];
	[rollCellTemplate release];
	
	return table;
}

@end

@implementation DXLogWindowController (Fetching)

/*- (BOOL)fetchLogEntries {
	BOOL success = NO;

	NSManagedObjectContext *moc = [[NSApp delegate] managedObjectContext];
	
	NSError *fetchError = nil;
	NSFetchRequest *fetchRequest = [[NSFetchRequest alloc] init];
	@try {
		NSEntityDescription *entity = [NSEntityDescription entityForName:@"DieNode"
			inManagedObjectContext:moc];
		[fetchRequest setEntity:entity];
		
		NSPredicate *predicate = [NSPredicate predicateWithFormat:@"parent == nil"];
		[fetchRequest setPredicate:predicate];
		
		[fetchRequest setSortDescriptors:[self logSortDescriptors]];

		success = [logTreeController fetchWithRequest:fetchRequest merge:YES error:&fetchError];
	} @finally {
		[fetchRequest release];
	}

	if (fetchError != nil) {
		[[NSApp delegate] presentError:fetchError];
	}
	
	return success;
}*/

- (void)updateFetchedBaseNodesCount {
	[self setFetchedBaseNodesCount:[[self fetchBaseNodes] count]];
}

- (NSArray *)fetchBaseNodes {
	NSArray *foundNodes = nil;

	NSManagedObjectContext *moc = [[NSApp delegate] managedObjectContext];
	//NSManagedObjectContext *moc = logManagedObjectContext;
	
	NSError *fetchError = nil;
	NSArray *fetchResults;
	NSFetchRequest *fetchRequest = [[NSFetchRequest alloc] init];
	@try {
		NSEntityDescription *entity = [NSEntityDescription entityForName:@"DieNode"
			inManagedObjectContext:moc];
		[fetchRequest setEntity:entity];
		
		NSPredicate *predicate = [NSPredicate predicateWithFormat:@"parent == nil"];
		[fetchRequest setPredicate:predicate];
		
		[fetchRequest setSortDescriptors:[self logSortDescriptors]];
		
		fetchResults = [moc executeFetchRequest:fetchRequest error:&fetchError];
	} @finally {
		[fetchRequest release];
	}

	if ((fetchResults != nil) /*&& ([fetchResults count] == 1)*/ && (fetchError == nil)) {
		foundNodes = fetchResults;
	} else if (fetchError != nil) {
		[[NSApp delegate] presentError:fetchError];
	}
	
	return foundNodes;
}

- (unsigned int)fetchedBaseNodesCount {
	return AM_fetchedBaseNodesCount;
}

- (void)setFetchedBaseNodesCount:(unsigned int)anInt {
	AM_fetchedBaseNodesCount = anInt;
}

@end

@implementation NSOutlineView (DXLogAdditions)

- (void)expandFirstItem:(id)sender {
	int rows = [self numberOfRows];
	if (rows > 0) {
		[self expandItem:[self itemAtRow:0]];
	}
}

- (void)expandLastItem:(id)sender {
	int rows = [self numberOfRows];
	if (rows > 0) {
		[self expandItem:[self itemAtRow:(rows - 1)]];
	}
}

- (void)scrollFirstRowToVisible:(id)sender {
	int rows = [self numberOfRows];
	if (rows > 0) {
		[self scrollRowToVisible:0];
	}
}

- (void)scrollLastRowToVisible:(id)sender {
	int rows = [self numberOfRows];
	if (rows > 0) {
		[self scrollRowToVisible:(rows - 1)];
	}
}

- (void)collapseAllRows:(id)sender {	
	unsigned int rows = [self numberOfRows];
	id item;
	
	int i;
	for (i = rows - 1; i >= 0; --i) {
		item = [self itemAtRow:i];
		if ([self isExpandable:item]) {
			[self collapseItem:item];
		}
	}
}

- (void)expandAllRows:(id)sender {
	unsigned int rows = [self numberOfRows];
	id item;
	
	int i;
	for (i = rows - 1; i >= 0; --i) {
		item = [self itemAtRow:i];
		if ([self isExpandable:item]) {
			[self expandItem:item];
		}
	}
}

@end


@implementation DXLogWindowController (Accessors)

- (NSArray *)logSortDescriptors {
	return [[AM_logSortDescriptors retain] autorelease];
}

- (void)setLogSortDescriptors:(NSArray *)anArray {
	/* do nothing */
	
	/*if (anArray != AM_logSortDescriptors) {
		[AM_logSortDescriptors release];
		AM_logSortDescriptors = anArray;
	}*/
}

- (int)lastMasterOrderHint {
	return AM_lastMasterOrderHint;
}

- (void)setLastMasterOrderHint:(int)anInt {
	AM_lastMasterOrderHint = anInt;
}

- (BOOL)isProcessingRolls {
	return AM_processingRolls;
}

- (void)setProcessingRolls:(BOOL)flag {
	AM_processingRolls = flag;
	[[NSNotificationCenter defaultCenter] postNotificationName:@"AMModifiedRollProcessingStatus" object:nil];
}

- (BOOL)isSaving {
	return AM_saving;
}

- (void)setSaving:(BOOL)flag {
	AM_saving = flag;
}

- (NSPersistentStoreCoordinator *)persistentStoreCoordinatorRef {
	return AM_psCoordinator;
}

- (void)setPersistentStoreCoordinatorRef:(NSPersistentStoreCoordinator *)coordinator {
	AM_psCoordinator = coordinator;
}

@end
