//
//  DXPreferencesWindowController.m
//  DiceX
//
//  Created by Andrew Merenbach on 3/9/06.
//  Copyright 2006 Andrew Merenbach. All rights reserved.
//

#import "DiceX_AppDelegate.h"
#import "DXPreferencesWindowController.h"
#import "DXAspectControllerMain.h"
#import "DXAspectControllerBasic.h"
#import "DXAspectControllerModifiers.h"
#import "DXAspectControllerRerolling.h"
#import "DXAspectControllerDrops.h"

#import "DXLogWindowController.h"

#import "Sparkle/SUConstants.h"

NSString *AMVersionKey = @"lastOpenedVersion";
NSString *AMShowLogOnStartupKey = @"showLogWindowAtStartup";


@implementation DXPreferencesWindowController

+ (void)initialize {
	NSDictionary *initialValues;
	
	initialValues = [[NSDictionary alloc] initWithObjectsAndKeys:
		[NSNumber numberWithBool:YES], SUCheckAtStartupKey,	/* check automatically for updates */
		[NSNumber numberWithBool:NO], AMShowLogOnStartupKey,	/* show (or don't show) log at startup */
		nil];
	[[NSUserDefaultsController sharedUserDefaultsController] setInitialValues:initialValues];
	[initialValues release];
	
	initialValues = [[NSDictionary alloc] initWithObjectsAndKeys:
		[NSNumber numberWithBool:YES], AMLogRollsKey,
		[NSNumber numberWithBool:YES], AMAutoscrollLogEntriesKey,
		[NSNumber numberWithBool:NO], AMAutoExpandLogEntriesKey,
		
		[NSNumber numberWithUnsignedInt:1], AMRollRepetitionsKey,
		//[NSNumber numberWithBool:YES], @"ascendingLogEntryOrder",

		// basic
		[NSNumber numberWithUnsignedInt:1], AMCountOfDiceKey,
		[NSNumber numberWithUnsignedInt:2], AMSidesPerDieKey,
		[NSNumber numberWithBool:NO], AMWillRollAsPercentileKey,
		[NSNumber numberWithBool:YES], AMValueForPercentileEqualsMaxKey,
		
		// modifiers
		[NSNumber numberWithBool:NO], AMWillApplyPerDieBonusValueKey,
		[NSNumber numberWithBool:NO], AMWillApplyPerDiePenaltyValueKey,
		[NSNumber numberWithBool:NO], AMWillApplyPerDieMultiplierValueKey,
		[NSNumber numberWithBool:NO], AMWillApplyToTotalBonusValueKey,
		[NSNumber numberWithBool:NO], AMWillApplyToTotalPenaltyValueKey,
		[NSNumber numberWithBool:NO], AMWillApplyToTotalMultiplierValueKey,
		
		[NSNumber numberWithUnsignedInt:0], AMPerDieBonusValueKey,
		[NSNumber numberWithUnsignedInt:0], AMPerDiePenaltyValueKey,
		[NSNumber numberWithUnsignedInt:0], AMPerDieMultiplierValueKey,
		[NSNumber numberWithUnsignedInt:0], AMToTotalBonusValueKey,
		[NSNumber numberWithUnsignedInt:0], AMToTotalPenaltyValueKey,
		[NSNumber numberWithUnsignedInt:0], AMToTotalMultiplierValueKey,
		
		// rerolling
		[NSNumber numberWithBool:NO], AMWillRerollIndividualDiceUnderValueKey,
		[NSNumber numberWithBool:NO], AMWillRerollIndividualDiceOverValueKey,
		[NSNumber numberWithInt:0], AMRerollIndividualDiceUnderValueKey,
		[NSNumber numberWithInt:0], AMRerollIndividualDiceOverValueKey,
		[NSNumber numberWithInt:0], AMRerollIndividualDicePrecedenceKey,
		
		[NSNumber numberWithBool:NO], AMWillRerollPercentileDiceUnderValueKey,
		[NSNumber numberWithBool:NO], AMWillRerollPercentileDiceOverValueKey,
		[NSNumber numberWithInt:0], AMRerollPercentileDiceUnderValueKey,
		[NSNumber numberWithInt:0], AMRerollPercentileDiceOverValueKey,
		[NSNumber numberWithInt:0], AMRerollPercentileDicePrecedenceKey,
		
		// drops
		[NSNumber numberWithBool:NO], AMWillDropLowestValueKey,
		[NSNumber numberWithBool:NO], AMWillDropHighestValueKey,
		[NSNumber numberWithUnsignedInt:0], AMLowestDropValueKey,
		[NSNumber numberWithUnsignedInt:0], AMHighestDropValueKey,

		nil];
	
	[[NSUserDefaults standardUserDefaults] registerDefaults:initialValues];
	[initialValues release];
}

- (id)init {
	self = [super initWithWindowNibName:@"Preferences"];
	return self;
}

@end
