//
//  DiceX_AppDelegate.m
//  DiceX
//
//  Created by Andrew Merenbach on 27/12/06.
//  Copyright Andrew Merenbach 2006 . All rights reserved.
//

#import "DiceX_AppDelegate.h"
#import "DXToolbarFactory.h"

#import "DXLogWindowController.h"
#import "DXLogWindowController+EntryManagement.h"
#import "DXPreferencesWindowController.h"	/* for AMShowLogOnStartupKey */

#import "ZDSMigrationHandler.h"
#import "NSWindow+AMAdditions.h"

#import "DXDieGroup.h"
#import "DXDie.h"

#import "DXAspectController.h"
#import "DXAspectControllerMain.h"
#import "DXAspectControllerBasic.h"
#import "DXAspectControllerModifiers.h"
#import "DXAspectControllerDrops.h"
#import "DXAspectControllerRerolling.h"


@implementation DiceX_AppDelegate


/**
	Returns the support folder for the application, used to store the Core Data
	store file.  This code uses a folder named "DiceX" for
	the content, either in the NSApplicationSupportDirectory location or (if the
	former cannot be found), the system's temporary directory.
 */

- (NSString *)applicationSupportFolder {

	/*NSArray *paths = NSSearchPathForDirectoriesInDomains(NSApplicationSupportDirectory, NSUserDomainMask, YES);
	NSString *basePath = ([paths count] > 0) ? [paths objectAtIndex:0] : NSTemporaryDirectory();
	return [basePath stringByAppendingPathComponent:@"DiceX"];*/
	
	NSString *str = [[NSProcessInfo processInfo] globallyUniqueString];	
    NSString *temp = [NSTemporaryDirectory() stringByAppendingPathComponent:str];
	
    return temp;
}


/**
	Creates, retains, and returns the managed object model for the application 
	by merging all of the models found in the application bundle and all of the 
	framework bundles.
 */
 
- (NSManagedObjectModel *)managedObjectModel {

	if (managedObjectModel == nil) {
		NSMutableSet *allBundles = [[NSMutableSet alloc] init];
		[allBundles addObject: [NSBundle mainBundle]];
		[allBundles addObjectsFromArray: [NSBundle allFrameworks]];
		
		managedObjectModel = [[NSManagedObjectModel mergedModelFromBundles: [allBundles allObjects]] retain];
		[allBundles release];
	}
	
	return managedObjectModel;
}

/* ZDS */
/*- (NSManagedObjectModel*)managedObjectModel;
{
    if (managedObjectModel) return managedObjectModel;

    NSString *modelFilePath = [[NSBundle mainBundle] pathForResource:@"DiceX_DataModel" ofType:@"mom"];
    NSURL *modelURL = [NSURL fileURLWithPath:modelFilePath];
    managedObjectModel = [[NSManagedObjectModel alloc] initWithContentsOfURL:modelURL];
    
    return managedObjectModel;
}*/

/**
	Returns the persistent store coordinator for the application.  This 
	implementation will create and return a coordinator, having added the 
	store for the application to it.  (The folder for the store is created, 
	if necessary.)
 */

- (NSPersistentStoreCoordinator *) persistentStoreCoordinator {

	if (persistentStoreCoordinator == nil) {
		NSFileManager *fileManager;
		NSString *applicationSupportFolder = nil;
		NSURL *url;
		NSError *error;
		
		fileManager = [NSFileManager defaultManager];
		applicationSupportFolder = [self applicationSupportFolder];
		if ( ![fileManager fileExistsAtPath:applicationSupportFolder isDirectory:NULL] ) {
			[fileManager createDirectoryAtPath:applicationSupportFolder attributes:nil];
		}
		
		url = [NSURL fileURLWithPath: [applicationSupportFolder stringByAppendingPathComponent: @"DiceX.sqlite"]];
		persistentStoreCoordinator = [[NSPersistentStoreCoordinator alloc] initWithManagedObjectModel: [self managedObjectModel]];
		if (![persistentStoreCoordinator addPersistentStoreWithType:NSSQLiteStoreType configuration:nil URL:url options:nil error:&error]){
			[[NSApplication sharedApplication] presentError:error];
		}
	}

	return persistentStoreCoordinator;
}


/**
	Returns the managed object context for the application (which is already
	bound to the persistent store coordinator for the application.) 
 */
 
- (NSManagedObjectContext *) managedObjectContext {

	if (managedObjectContext == nil) {
		NSPersistentStoreCoordinator *coordinator = [self persistentStoreCoordinator];
		if (coordinator != nil) {
			managedObjectContext = [[NSManagedObjectContext alloc] init];
			[managedObjectContext setPersistentStoreCoordinator: coordinator];
		}
	}
	
	return managedObjectContext;
}


/**
	Returns the NSUndoManager for the application.  In this case, the manager
	returned is that of the managed object context for the application.
 */
 
- (NSUndoManager *)windowWillReturnUndoManager:(NSWindow *)window {
	return [[self managedObjectContext] undoManager];
}


/**
	Performs the save action for the application, which is to send the save:
	message to the application's managed object context.  Any encountered errors
	are presented to the user.
 */
 
- (IBAction) saveAction:(id)sender {

	NSError *error = nil;
	if (![[self managedObjectContext] save:&error]) {
		[[NSApplication sharedApplication] presentError:error];
	}
}


/**
	Implementation of the applicationShouldTerminate: method, used here to
	handle the saving of changes in the application managed object context
	before the application terminates.
 */
 
- (NSApplicationTerminateReply)applicationShouldTerminate:(NSApplication *)sender {

	NSError *error;
	int reply = NSTerminateNow;
	
	if (managedObjectContext != nil) {
		if ([managedObjectContext commitEditing]) {
			if ([managedObjectContext hasChanges] && ![managedObjectContext save:&error]) {
				
				// This error handling simply presents error information in a panel with an 
				// "Ok" button, which does not include any attempt at error recovery (meaning, 
				// attempting to fix the error.)  As a result, this implementation will 
				// present the information to the user and then follow up with a panel asking 
				// if the user wishes to "Quit Anyway", without saving the changes.

				// Typically, this process should be altered to include application-specific 
				// recovery steps.  

				BOOL errorResult = [[NSApplication sharedApplication] presentError:error];
				
				if (errorResult == YES) {
					reply = NSTerminateCancel;
				} 

				else {
					
					int alertReturn = NSRunAlertPanel(nil, @"Could not save changes while quitting. Quit anyway?" , @"Quit anyway", @"Cancel", nil);
					if (alertReturn == NSAlertAlternateReturn) {
						reply = NSTerminateCancel;	
					}
				}
			}
		} 
		
		else {
			reply = NSTerminateCancel;
		}
	}

	{
		BOOL isSavingInAnotherThread;
		NSAlert *alert;
		
		if (isSavingInAnotherThread = [logWindowController isSaving]) {
			reply = NSTerminateCancel;
			
			alert = [NSAlert alertWithMessageText:@"Cannot Quit"
					defaultButton:@"Cancel"
					alternateButton:nil
					otherButton:nil
					informativeTextWithFormat:@"The program is in the process of saving your settings.  Please try to quit again in a few moments."];
			[alert setAlertStyle:NSInformationalAlertStyle];
			[alert beginSheetModalForWindow:window modalDelegate:nil didEndSelector:NULL contextInfo:nil];
			
		} else if (isSavingInAnotherThread = [mainAspectController isSaving]) {
			reply = NSTerminateCancel;
			
			alert = [NSAlert alertWithMessageText:@"Cannot Quit"
					defaultButton:@"Cancel"
					alternateButton:nil
					otherButton:nil
					informativeTextWithFormat:@"The program is in the process of saving your settings.  Please try to quit again in a few moments."];
			[alert setAlertStyle:NSInformationalAlertStyle];
			[alert beginSheetModalForWindow:window modalDelegate:nil didEndSelector:NULL contextInfo:nil];
		}
	}
	
	return reply;
}

- (id)init {
	if (nil != (self = [super init])) {
		[self setSummary:[[[NSAttributedString alloc] initWithString:@""] autorelease]];	//AM_summary = [[NSString alloc] init];
		[self setRolling:NO];

		NSZone *thisZone = [self zone];
		
		aspectControllers = [[NSMutableDictionary allocWithZone:thisZone] init];
				
		id aspectControllerMain = [[DXAspectControllerMain allocWithZone:thisZone] init];
		id aspectControllerBasic = [[DXAspectControllerBasic allocWithZone:thisZone] init];
		id aspectControllerModifiers = [[DXAspectControllerModifiers allocWithZone:thisZone] init];
		id aspectControllerDrops = [[DXAspectControllerDrops allocWithZone:thisZone] init];
		id aspectControllerRerolling = [[DXAspectControllerRerolling allocWithZone:thisZone] init];
		
		[aspectControllers setObject:aspectControllerMain forKey:[aspectControllerMain identifier]];
		[aspectControllers setObject:aspectControllerBasic forKey:[aspectControllerBasic identifier]];
		[aspectControllers setObject:aspectControllerModifiers forKey:[aspectControllerModifiers identifier]];
		[aspectControllers setObject:aspectControllerDrops forKey:[aspectControllerDrops identifier]];
		[aspectControllers setObject:aspectControllerRerolling forKey:[aspectControllerRerolling identifier]];
		
		[aspectControllerMain release];
		[aspectControllerBasic release];
		[aspectControllerModifiers release];
		[aspectControllerDrops release];
		[aspectControllerRerolling release];
		
		[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(updateSummary:) name:@"AMModifiedSettings" object:nil];
		[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(updateTitleForRollButton:) name:@"AMModifiedRollProcessingStatus" object:nil];
		
		blankView = [[NSView alloc] init];
	}
	return self;
}

/**
	Implementation of dealloc, to release the retained variables.
 */
 
- (void) dealloc {
	mainAspectController = nil;	// reset link

	[self setSummary:nil];
	
	[aspectControllers release];
	aspectControllers = nil;
	
	[[NSNotificationCenter defaultCenter] removeObserver:self name:@"AMModifiedSettings" object:nil];
	[[NSNotificationCenter defaultCenter] removeObserver:self name:@"AMModifiedRollProcessingStatus" object:nil];
	
	[blankView release];
	blankView = nil;
	
	/* Core Data stuff */
	[managedObjectContext release], managedObjectContext = nil;
	[persistentStoreCoordinator release], persistentStoreCoordinator = nil;
	[managedObjectModel release], managedObjectModel = nil;
	[super dealloc];
}

- (void)awakeFromNib {
	mainAspectController = [aspectControllers objectForKey:@"Main"];

	[self loadAllValuesFromDefaults];

	[toolbarFactory setupToolbarWithIdentifier:@"MainToolbar"];
	
	[self updateSummary:nil];
	[self loadAspectMain:nil];
}

- (BOOL)applicationShouldTerminateAfterLastWindowClosed:(NSApplication *)theApplication {
	return YES;
}

- (void)applicationDidFinishLaunching:(NSNotification *)notification {
	/* we want a non-lazy loading of the log window
		so that auto-expand will work properly */
	(void)[logWindowController window];
	
	// open the log window at startup if specified in user preferences
	if ([[[[NSUserDefaultsController sharedUserDefaultsController] values] valueForKey:AMShowLogOnStartupKey] boolValue]) {
	//if ([[NSUserDefaults standardUserDefaults] boolForKey:AMShowLogOnStartupKey])
		[logWindowController showWindow:nil];
	}

	// for logging and adding rolls to the roll table view
	id psCoordinator = [self persistentStoreCoordinator];
	[logWindowController setPersistentStoreCoordinatorRef:psCoordinator];
	[mainAspectController setPersistentStoreCoordinatorRef:psCoordinator];
}

- (void)applicationWillTerminate:(NSNotification *)notification {
	[self saveAllValuesToDefaults];
}

/* ZDS */
/*- (void)applicationDidFinishLaunching:(NSNotification *)notification;
{
    NSManagedObjectContext *moc = [self managedObjectContext];
    NSAssert(moc != nil, @"MOC is nil");
    //Check to see if there is any data in this database
    NSManagedObjectModel *mom = [self managedObjectModel];
    NSFetchRequest *fetch = [[[NSFetchRequest alloc] init] autorelease];
    [fetch setEntity:[[mom entities] objectAtIndex:0]];
    NSError *error;
    NSArray *results = [moc executeFetchRequest:fetch error:&error];
    NSAssert(results != nil, ([NSString stringWithFormat:@"Error on fetch: %@", error]));
    if ([results count] > 0) return; //There is already data in place
    //[self performSelector:@selector(populateDatabase) withObject:nil afterDelay:0.01];
}*/

/* ZDS */
/*- (IBAction)startMigration:(id)sender;
{
    //Find the new model path
    NSString *newModelFilePath = [[NSBundle mainBundle] pathForResource:@"DiceX_DataModel" ofType:@"mom"];
    NSString *oldModelFilePath = [[NSBundle mainBundle] pathForResource:@"DiceX_DataModel" ofType:@"mom"];
    
    ZDSMigrationHandler *handler = [[ZDSMigrationHandler alloc] initWithDelegate:self];
    [handler setPathForModelToMigrateFrom:oldModelFilePath];
    [handler setPathForModelToMigrateTo:newModelFilePath];
    [handler setPathForFileToMigrate:[[self applicationSupportFolder] stringByAppendingPathComponent:@"Test.zds"]];
    [handler setWarnings:YES];
    [handler setThreaded:YES];
    [handler setMigrationHelper:[[[ExampleMigrationHelper alloc] init] autorelease]];
    [handler startMigration];
}*/

- (void)loadAllValuesFromDefaults {
	// we don't want to do the following block yet, since we're using the XML data store type
	/*{
		NSBundle *bundle = [NSBundle mainBundle];
		NSDictionary *infoDict = [bundle infoDictionary];
		NSString *currentVersion = [infoDict objectForKey:(NSString *)kCFBundleVersionKey];
	
		NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
		
		NSString *lastOpenedVersion = [userDefaults objectForKey:AMVersionKey];
		if (![currentVersion isEqualToString:lastOpenedVersion]) {
			[self startMigration:nil];
		}
	}*/
	
	NSArray *controllers = [aspectControllers allValues];
	[controllers makeObjectsPerformSelector:@selector(loadValuesFromDefaults) withObject:nil];
	[[aspectControllers objectForKey:@"Basic"] modifiedPercentileRoll:nil];
	//[logWindowController loadValuesFromDefaults:self];
	//[self setSummary:[[[NSUserDefaultsController sharedUserDefaultsController] values] valueForKey:AMSummaryKey]];
}

- (void)saveAllValuesToDefaults {
	NSArray *controllers = [aspectControllers allValues];
	[controllers makeObjectsPerformSelector:@selector(saveValuesToDefaults) withObject:nil];
	//[logWindowController saveValuesToDefaults:self];
	
	{
		NSBundle *bundle = [NSBundle mainBundle];
		NSDictionary *infoDict = [bundle infoDictionary];
		NSString *currentVersion = [infoDict objectForKey:(NSString *)kCFBundleVersionKey];
		
		NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
		
		NSString *lastOpenedVersion = [userDefaults valueForKey:AMVersionKey];
		
		if (![currentVersion isEqualToString:lastOpenedVersion]) {
			[userDefaults setObject:currentVersion forKey:AMVersionKey];
		}
	}
}

- (IBAction)launchThreadForDiceRoll:(id)sender {
	//NSLog(@"[DiceX] multi-threading disabled for rolling");
	BOOL isRolling = [self isRolling];
	BOOL logIsProcessingRolls = [logWindowController isProcessingRolls];
	BOOL mainAspectIsProcessingRolls = [mainAspectController isProcessingRolls];
	
	BOOL rollsBeingDone = (isRolling || logIsProcessingRolls || mainAspectIsProcessingRolls);

	[self setRolling:!rollsBeingDone];
	[mainAspectController setProcessingRolls:!rollsBeingDone];
	[logWindowController setProcessingRolls:!rollsBeingDone];
	
	if (!rollsBeingDone) {
		[NSThread detachNewThreadSelector:@selector(roll:) toTarget:self withObject:sender];
	}
	//[self roll:sender];
}

- (void)roll:(id)sender {
	NSAutoreleasePool *pool = [[NSAutoreleasePool allocWithZone:[self zone]] init];
	
	//id mainAspectController = [aspectControllers objectForKey:@"Main"];
	id basicAspectController = [aspectControllers objectForKey:@"Basic"];
	id modifiersAspectController = [aspectControllers objectForKey:@"Modifiers"];
	id rerollingAspectController = [aspectControllers objectForKey:@"Rerolling"];
	id dropsAspectController = [aspectControllers objectForKey:@"Drops"];

		/* basic */
		unsigned int countOfDice = [basicAspectController countOfDice];
		unsigned int sidesPerDie = [basicAspectController sidesPerDie];
		BOOL willRollAsPercentile = [basicAspectController willRollAsPercentile];
		BOOL willPercentileZeroEqualMax = [basicAspectController percentileZeroEqualsMax];
		//unsigned int valueForPercentileZeroEqualsMax = [basicAspectController valueForPercentileZeroEqualsMax];
		
		/* modifiers */
		BOOL willApplyPerDieBonusValue = [modifiersAspectController willApplyPerDieBonusValue];
		BOOL willApplyPerDiePenaltyValue = [modifiersAspectController willApplyPerDiePenaltyValue];
		BOOL willApplyPerDieMultiplierValue = [modifiersAspectController willApplyPerDieMultiplierValue];
		
		BOOL willApplyToTotalBonusValue = [modifiersAspectController willApplyToTotalBonusValue];
		BOOL willApplyToTotalPenaltyValue = [modifiersAspectController willApplyToTotalPenaltyValue];
		BOOL willApplyToTotalMultiplierValue = [modifiersAspectController willApplyToTotalMultiplierValue];
		
		unsigned int perDieBonusValue = [modifiersAspectController perDieBonusValue];
		unsigned int perDiePenaltyValue = [modifiersAspectController perDiePenaltyValue];
		unsigned int perDieMultiplierValue = [modifiersAspectController perDieMultiplierValue];

		unsigned int toTotalBonusValue = [modifiersAspectController toTotalBonusValue];
		unsigned int toTotalPenaltyValue = [modifiersAspectController toTotalPenaltyValue];
		unsigned int toTotalMultiplierValue = [modifiersAspectController toTotalMultiplierValue];
		
		/* rerolling */
		BOOL willRerollIndividualDiceUnderValue = [rerollingAspectController willRerollIndividualDiceUnderValue];
		BOOL willRerollIndividualDiceOverValue = [rerollingAspectController willRerollIndividualDiceOverValue];
		unsigned int rerollIndividualDiceUnderValue = [rerollingAspectController rerollIndividualDiceUnderValue];
		unsigned int rerollIndividualDiceOverValue = [rerollingAspectController rerollIndividualDiceOverValue];
		AMRerollPrecedence rerollIndividualDicePrecedence = [rerollingAspectController rerollIndividualDicePrecedence];
		//BOOL rerollingIndividualDiceOptionsEnabled = [rerollingAspectController rerollingIndividualDiceOptionsEnabled];
		
		BOOL willRerollPercentileDiceUnderValue = [rerollingAspectController willRerollPercentileDiceUnderValue];
		BOOL willRerollPercentileDiceOverValue = [rerollingAspectController willRerollPercentileDiceOverValue];
		unsigned int rerollPercentileDiceUnderValue = [rerollingAspectController rerollPercentileDiceUnderValue];
		unsigned int rerollPercentileDiceOverValue = [rerollingAspectController rerollPercentileDiceOverValue];
		AMRerollPrecedence rerollPercentileDicePrecedence = [rerollingAspectController rerollPercentileDicePrecedence];
		//BOOL rerollingPercentileDiceOptionsEnabled = [rerollingAspectController rerollingPercentileDiceOptionsEnabled];
		
		/* drops */
		BOOL willDropLowest = [dropsAspectController willDropLowestValue];
		BOOL willDropHighest = [dropsAspectController willDropHighestValue];
		unsigned int numberOfLowDrops = [dropsAspectController lowestDropValue];
		unsigned int numberOfHighDrops = [dropsAspectController highestDropValue];
		
		
		/* conditional values */
		unsigned int conditionalPerDieBonusValue = (willApplyPerDieBonusValue ? perDieBonusValue : 0);
		unsigned int conditionalPerDiePenaltyValue = (willApplyPerDiePenaltyValue ? perDiePenaltyValue : 0);
		unsigned int conditionalToTotalBonusValue = (willApplyToTotalBonusValue ? toTotalBonusValue : 0);
		unsigned int conditionalToTotalPenaltyValue = (willApplyToTotalPenaltyValue ? toTotalPenaltyValue : 0);
		
		unsigned int conditionalLowDrops = (willDropLowest ? numberOfLowDrops : 0);
		unsigned int conditionalHighDrops = (willDropHighest ? numberOfHighDrops : 0);
		
		/* multi-roll */
		unsigned int rollRepetitions = [mainAspectController rollRepetitions];
		
		//unsigned int totalDrops = lowDrops + highDrops;
		if (rollRepetitions < 0) {
			[self performSelectorOnMainThread:@selector(alertNegativeRollRepetitions) withObject:nil waitUntilDone:NO];
			return;
			
		} else if (conditionalLowDrops + conditionalHighDrops >= countOfDice && countOfDice > 0) {
			[self performSelectorOnMainThread:@selector(alertCannotDropAllDice) withObject:nil waitUntilDone:NO];
			return;
		} /*else if (countOfDice == 0) {
			alert = [NSAlert alertWithMessageText:@"Quantity Error"
					defaultButton:@"Cancel"
					alternateButton:nil
					otherButton:nil
					informativeTextWithFormat:@"You must roll at least one die."];
			[alert setAlertStyle:NSInformationalAlertStyle];
			[alert beginSheetModalForWindow:window modalDelegate:nil didEndSelector:NULL contextInfo:nil];
			
			return;
			
		}*/ else if (willRerollIndividualDiceOverValue && rerollIndividualDiceOverValue < (1 /* min sides per die */ + conditionalPerDieBonusValue - conditionalPerDiePenaltyValue)) {
			[self performSelectorOnMainThread:@selector(alertRerollingRangeErrorMinimum) withObject:nil waitUntilDone:NO];
			return;
			
		} else if (willRerollIndividualDiceUnderValue && rerollIndividualDiceUnderValue > (sidesPerDie + conditionalPerDieBonusValue - conditionalPerDiePenaltyValue)) {
			[self performSelectorOnMainThread:@selector(alertRerollingRangeErrorMaximum) withObject:nil waitUntilDone:NO];	
			return;
			
		} else if (willRollAsPercentile) {
			unsigned int max = [basicAspectController valueForPercentileZeroEqualsMax];
			unsigned int min = countOfDice;

			if (!willPercentileZeroEqualMax) {
				--min;
				--max;
			}
			
			if (willRerollPercentileDiceOverValue && rerollPercentileDiceOverValue < (min + conditionalToTotalBonusValue - conditionalToTotalPenaltyValue)) {
				[self performSelectorOnMainThread:@selector(alertPercentileRerollingRangeErrorMinimum) withObject:nil waitUntilDone:NO];	
				return;
				
			} else if (willRerollPercentileDiceUnderValue && rerollPercentileDiceUnderValue > (max + conditionalToTotalBonusValue - conditionalToTotalPenaltyValue)) {
				[self performSelectorOnMainThread:@selector(alertPercentileRerollingRangeErrorMaximum) withObject:nil waitUntilDone:NO];	
				return;
				
			}
		} else if (countOfDice < 0 || sidesPerDie < 0
				|| perDieBonusValue < 0 || perDiePenaltyValue < 0
				|| toTotalBonusValue < 0 || toTotalPenaltyValue < 0
				|| numberOfLowDrops < 0 || numberOfHighDrops < 0)
		{
			/* we're using formatters, so this is mainly a sanity-check */
			[self performSelectorOnMainThread:@selector(alertNegativeValues) withObject:nil waitUntilDone:NO];
			return;
		}
		
		//[logWindowController setCountOfNewNodes:0];	// old "new nodes" are no longer new; so even if not logging, set to zeor
		//[mainAspectController performSelectorOnMainThread:@selector(clearRolls) withObject:nil waitUntilDone:YES];
		[mainAspectController clearRolls];
		
		NSZone *thisZone = [self zone];
		
		NSMutableArray *rollArray = [[NSMutableArray alloc] initWithCapacity:rollRepetitions];
		
		int i;
		for (i = 0; i < rollRepetitions && [self isRolling]; ++i) {
			/* and now, for the rolling logic */
			DXDieGroup *dieGroup = [[DXDieGroup allocWithZone:thisZone] init];

			[dieGroup setDieCount:countOfDice];
			[dieGroup setSidesPerDie:sidesPerDie];
			
			if (!willRollAsPercentile) {
				[dieGroup rollDice];
				
				/* reroll dice not initially fitting within the proper range */
				if (rerollIndividualDicePrecedence == AMRerollBeforeModifiers) {
					if (willRerollIndividualDiceUnderValue) {
						[dieGroup rerollPercentileDiceWithValuesUnder:rerollIndividualDiceUnderValue withMods:NO];
					}

					if (willRerollIndividualDiceOverValue) {
						[dieGroup rerollPercentileDiceWithValuesOver:rerollIndividualDiceOverValue withMods:NO];
					}
				}
				
				[dieGroup tallyDice];
				
				if (willApplyPerDieBonusValue) {
					[dieGroup setPerDieBonusModifier:perDieBonusValue];
				}
				if (willApplyPerDiePenaltyValue) {
					[dieGroup setPerDiePenaltyModifier:perDiePenaltyValue];
				}
				if (willApplyPerDieMultiplierValue) {
					[dieGroup setPerDieMultiplierModifier:perDieMultiplierValue];
				}
				
				[dieGroup tallyDice];
				
				if (rerollIndividualDicePrecedence == AMRerollAfterModifiers) {
					if (willRerollIndividualDiceUnderValue) {
						[dieGroup rerollPercentileDiceWithValuesUnder:rerollIndividualDiceUnderValue withMods:YES];
					}

					if (willRerollIndividualDiceOverValue) {
						[dieGroup rerollPercentileDiceWithValuesOver:rerollIndividualDiceOverValue withMods:YES];
					}
				}
				
				[dieGroup sortDice];
				
				if (willDropLowest) {
					[dieGroup dropLowRollsWithCount:numberOfLowDrops];
				}
				if (willDropHighest) {
					[dieGroup dropHighRollsWithCount:numberOfHighDrops];
				}
				
				if (willApplyToTotalBonusValue) {
					[dieGroup setTotalBonusModifier:toTotalBonusValue];
				}
				if (willApplyToTotalPenaltyValue) {
					[dieGroup setTotalPenaltyModifier:toTotalPenaltyValue];
				}
				if (willApplyToTotalMultiplierValue) {
					[dieGroup setTotalMultiplierModifier:toTotalMultiplierValue];
				}
				
				[dieGroup tallyDice];
				
			} else {
				[dieGroup setIsPercentileRoll:YES];
				
				BOOL success;
				do {
					success = YES;
					
					[dieGroup rollDiceForPercentile];
					[dieGroup tallyDiceForPercentileRoll:willPercentileZeroEqualMax];
					
					if (rerollPercentileDicePrecedence == AMRerollBeforeModifiers) {
						if (willRerollPercentileDiceUnderValue && [dieGroup total] < rerollPercentileDiceUnderValue) {
							success = NO;
						}

						if (willRerollPercentileDiceOverValue && [dieGroup total] > rerollPercentileDiceOverValue) {
							success = NO;
						}
					}
					
					if (success) {
						if (willApplyToTotalBonusValue) {
							[dieGroup setTotalBonusModifier:toTotalBonusValue];
						}
						if (willApplyToTotalPenaltyValue) {
							[dieGroup setTotalPenaltyModifier:toTotalPenaltyValue];
						}
						if (willApplyToTotalMultiplierValue) {
							[dieGroup setTotalMultiplierModifier:toTotalMultiplierValue];
						}
						
						[dieGroup tallyDiceForPercentileRoll:willPercentileZeroEqualMax];
						
						if (rerollPercentileDicePrecedence == AMRerollBeforeModifiers) {
							if (willRerollPercentileDiceUnderValue && [dieGroup total] < rerollPercentileDiceUnderValue) {
								success = NO;
							}

							if (willRerollPercentileDiceOverValue && [dieGroup total] > rerollPercentileDiceOverValue) {
								success = NO;
							}
						}
					}

					
				} while (!success && [self isRolling]);
			}
			
			[rollArray addObject:dieGroup];
			
			[dieGroup release];
		}
		
		[self setRolling:NO];
		
		//[NSThread detachNewThreadSelector:@selector(addRolls:) toTarget:mainAspectController withObject:rollArray];
		[mainAspectController addRolls:rollArray];
		
		//[mainAspectController setProcessingRolls:NO];

		BOOL willLogRolls = [mainAspectController logRolls];
				
		if (willLogRolls) {
			BOOL scroll = [mainAspectController autoscrollLogEntries];
			BOOL expand = [mainAspectController autoExpandLogEntries];
			
			[logWindowController setAutoExpandEntries:expand];
			[logWindowController setAutoscrollEntries:scroll];
			
			//[NSThread detachNewThreadSelector:@selector(logRolls:) toTarget:logWindowController withObject:rollArray];
			[logWindowController logRolls:rollArray];
			//[logWindowController performSelectorOnMainThread:@selector(logRolls:) withObject:rollArray waitUntilDone:YES];
			//[logWindowController performSelectorOnMainThread:@selector(finishLoggingRoll:) withObject:rollArray waitUntilDone:YES];

		} else {
			[logWindowController setProcessingRolls:NO];
		}
		
		[rollArray release];
	
	[pool release];
}

/* use for both clearing log and for (voluntary) new version migration */
/*- (IBAction)resetInterface:(id)sender {
	NSAlert *alert = [NSAlert alertWithMessageText:@"Warning!"
			defaultButton:@"OK"
			alternateButton:@"Cancel"
			otherButton:nil
			informativeTextWithFormat:@"Are you sure that you want to reset all parameters and log entries?"];
	[alert setAlertStyle:NSWarningAlertStyle];	// NSCriticalAlertStyle
	[alert beginSheetModalForWindow:window modalDelegate:self
		didEndSelector:@selector(resetSheetDidEnd:returnCode:) contextInfo:NULL];	// contextInfo:
}

- (void)resetSheetDidEnd:(NSSavePanel *)sheet returnCode:(int)returnCode
{
	if (returnCode == NSOKButton) {
		[logWindowController updateFetchedBaseNodesCount];	// recaches things so that the log can
															// be cleared, even if it was never opened
	
		NSSet *regObjs = [managedObjectContext registeredObjects];
		NSEnumerator *e = [regObjs objectEnumerator];
		id obj;

		while (obj = [e nextObject]) {
			[managedObjectContext deleteObject:obj];
		}
		
		[[NSUserDefaultsController sharedUserDefaultsController] revertToInitialValues:self];
		[[aspectControllers allValues] makeObjectsPerformSelector:@selector(loadValuesFromDefaults:) withObject:self];
	}
}*/


- (void)showLogWindow:(id)sender {
	[logWindowController showWindow:sender];
}

- (void)toggleSummaryDrawer:(id)sender {
	[summaryDrawer toggle:sender];
}

- (void)updateSummary:(NSNotification *)aNotification {
	//id mainAspectController = [aspectControllers objectForKey:@"Main"];
	id basicAspectController = [aspectControllers objectForKey:@"Basic"];
	id modifiersAspectController = [aspectControllers objectForKey:@"Modifiers"];
	id rerollingAspectController = [aspectControllers objectForKey:@"Rerolling"];
	id dropsAspectController = [aspectControllers objectForKey:@"Drops"];
		
	/* multi-roll */
	unsigned int rollRepetitions = [mainAspectController rollRepetitions];	

	NSMutableArray *sentences = [[NSMutableArray alloc] init];
	NSString *sentence;
	
	NSString *header = @"Summary\n\n";
	
	{

		[sentences addObject:[(DXAspectController *)basicAspectController summary]];
		[sentences addObject:[(DXAspectController *)modifiersAspectController summary]];
		[sentences addObject:[(DXAspectController *)rerollingAspectController summary]];
		[sentences addObject:[(DXAspectController *)dropsAspectController summary]];
			
		if (rollRepetitions > 1) {
			sentence = [[NSString alloc] initWithFormat:@"(This process will be performed %i times.)", rollRepetitions];
			[sentences addObject:sentence];
			[sentence release];
		}
	}
	
	{
		NSDictionary *headerAttrs = [[NSDictionary alloc] initWithObjectsAndKeys:
			[NSFont boldSystemFontOfSize:12], NSFontAttributeName,
			[NSColor blackColor], NSForegroundColorAttributeName,
			nil];
		NSMutableAttributedString *summary = [[NSMutableAttributedString alloc] initWithString:header attributes:headerAttrs];
		[headerAttrs release];
		
		[sentences removeObject:@""];	// this may or may not be a kludge, but it's cleaner than testing each and every addition to [sentences] by hand
		NSString *concatenatedBody = [sentences componentsJoinedByString:@"\n\n"];
		
		NSDictionary *bodyAttrs = [[NSDictionary alloc] initWithObjectsAndKeys:
			[NSFont systemFontOfSize:12], NSFontAttributeName,
			[NSColor blackColor], NSForegroundColorAttributeName,
			nil];
		NSAttributedString *attrBody = [[NSAttributedString alloc] initWithString:concatenatedBody attributes:bodyAttrs];
		[bodyAttrs release];
		
		[summary appendAttributedString:attrBody];
		[attrBody release];
		
		[self setSummary:summary];

		[sentences release];
		[summary release];
	}
}

- (NSAttributedString *)summary {
	id obj;
	
	@synchronized(self) {
		obj = [[AM_summary retain] autorelease];
	}
	
	return obj;
}

- (void)setSummary:(NSAttributedString *)aString {
	@synchronized(AM_summary) {
		if (aString != AM_summary) {
			[AM_summary release];
			AM_summary = [aString copy];
		}
	}
}

- (BOOL)isRolling {
	return AM_rolling;
}

- (void)setRolling:(BOOL)flag {
	AM_rolling = flag;
	[[NSNotificationCenter defaultCenter] postNotificationName:@"AMModifiedRollProcessingStatus" object:nil];
}

- (void)updateTitleForRollButton:(NSNotification *)aNotification {
	BOOL isRolling = ([self isRolling] || [logWindowController isProcessingRolls] || [mainAspectController isProcessingRolls]);
	NSString *newTitle = (!isRolling ? @"Roll" : @"Stop");
	[mainAspectController setTitleForRollButton:newTitle];
}

@end

@implementation DiceX_AppDelegate (RollingAlerts)

/*- (void)displayAlertWithMessageText:(NSString *)messageText andInformativeText:(NSString *)informativeText {
	NSAlert *alert = [NSAlert alertWithMessageText:messageText
			defaultButton:@"Cancel"
			alternateButton:nil
			otherButton:nil
			informativeTextWithFormat:informativeText];
	[alert setAlertStyle:NSInformationalAlertStyle];
	[alert beginSheetModalForWindow:window modalDelegate:nil didEndSelector:NULL contextInfo:nil];
}*/

- (void)alertCannotDropAllDice {
	NSAlert *alert = [NSAlert alertWithMessageText:@"Quantity Error"
			defaultButton:@"Cancel"
			alternateButton:nil
			otherButton:nil
			informativeTextWithFormat:@"You cannot drop all of the dice that are being rolled."];
	[alert setAlertStyle:NSInformationalAlertStyle];
	[alert beginSheetModalForWindow:window modalDelegate:nil didEndSelector:NULL contextInfo:nil];
}

- (void)alertRerollingRangeErrorMinimum {
	NSAlert *alert = [NSAlert alertWithMessageText:@"Rerolling Range Error"
			defaultButton:@"Cancel"
			alternateButton:nil
			otherButton:nil
			informativeTextWithFormat:@"No die can yield a result less than its minimum result (1), plus the \"per die\" bonus modifier (if enabled), minus the \"per die\" penalty modifier (if enabled)."];
	[alert setAlertStyle:NSInformationalAlertStyle];
	[alert beginSheetModalForWindow:window modalDelegate:nil didEndSelector:NULL contextInfo:nil];
}

- (void)alertRerollingRangeErrorMaximum {
	NSAlert *alert = [NSAlert alertWithMessageText:@"Rerolling Range Error"
			defaultButton:@"Cancel"
			alternateButton:nil
			otherButton:nil
			informativeTextWithFormat:@"No die can yield a result greater than its maximum result (the number of sides), plus the \"per die\" bonus modifier (if enabled), minus the \"per die\" penalty modifier (if enabled)."];
	[alert setAlertStyle:NSInformationalAlertStyle];
	[alert beginSheetModalForWindow:window modalDelegate:nil didEndSelector:NULL contextInfo:nil];
}

- (void)alertPercentileRerollingRangeErrorMinimum {
	NSAlert *alert = [NSAlert alertWithMessageText:@"Rerolling Range Error"
			defaultButton:@"Cancel"
			alternateButton:nil
			otherButton:nil
			informativeTextWithFormat:@"Percentile rolls cannot yield a result less than their minimum value (the number of dice [minus one if zero does not equal the maximum]), plus the \"to total\" bonus modifier (if enabled), minus the \"to total\" penalty modifier (if enabled)."];
	[alert setAlertStyle:NSInformationalAlertStyle];
	[alert beginSheetModalForWindow:window modalDelegate:nil didEndSelector:NULL contextInfo:nil];
}

- (void)alertPercentileRerollingRangeErrorMaximum {
	NSAlert *alert = [NSAlert alertWithMessageText:@"Rerolling Range Error"
			defaultButton:@"Cancel"
			alternateButton:nil
			otherButton:nil
			informativeTextWithFormat:@"Percentile rolls cannot yield a result greater than their maximum value (the number of sides raised to the power of the number of dice [minus one if zero does not equal the maximum]), plus the \"to total\" bonus modifier (if enabled), minus the \"to total\" penalty modifier (if enabled)."];
	[alert setAlertStyle:NSInformationalAlertStyle];
	[alert beginSheetModalForWindow:window modalDelegate:nil didEndSelector:NULL contextInfo:nil];
}

- (void)alertNegativeRollRepetitions {
	NSAlert *alert = [NSAlert alertWithMessageText:@"Negative Repetitions Error"
			defaultButton:@"Cancel"
			alternateButton:nil
			otherButton:nil
			informativeTextWithFormat:@"You cannot roll for fewer than zero repetitions."];
	[alert setAlertStyle:NSInformationalAlertStyle];
	[alert beginSheetModalForWindow:window modalDelegate:nil didEndSelector:NULL contextInfo:nil];
}

/* we're using formatters, so this is mainly a sanity-check */
- (void)alertNegativeValues {
	NSAlert *alert = [NSAlert alertWithMessageText:@"Negative Value Error"
			defaultButton:@"Cancel"
			alternateButton:nil
			otherButton:nil
			informativeTextWithFormat:@"At least one of your settings is less than zero, but it's not supposed to be."];
	[alert setAlertStyle:NSInformationalAlertStyle];
	[alert beginSheetModalForWindow:window modalDelegate:nil didEndSelector:NULL contextInfo:nil];
}

@end

@implementation DiceX_AppDelegate (ToolbarAspectMethods)

- (void)loadAspectMain:(id)sender {
	[self loadAspectWithIdentifier:@"Main"];
}

- (void)loadAspectBasic:(id)sender {
	[self loadAspectWithIdentifier:@"Basic"];
}

- (void)loadAspectModifiers:(id)sender {
	[self loadAspectWithIdentifier:@"Modifiers"];
}

- (void)loadAspectDrops:(id)sender {
	[self loadAspectWithIdentifier:@"Drops"];
}

- (void)loadAspectRerolling:(id)sender {
	[self loadAspectWithIdentifier:@"Rerolling"];
}

- (void)loadAspectWithIdentifier:(NSString *)identifier {
	id controller = [aspectControllers objectForKey:identifier];
	id view = [controller view];

	/* the following line does nothing when command is dispatched from toolbar,
		but does have an effect when the command is dispatched from the menu */
	[[window toolbar] setSelectedItemIdentifier:[controller identifier]];
	
	/* swap the content view of the main window */
	[window setContentView:blankView];
		[window setTitle:@"DiceX"];
	[window resizeToSize:[view frame].size];
		[window setTitle:[NSString stringWithFormat:@"DiceX (%@)", [controller label]]];
	[window setContentView:view];
}

@end


/*@interface DXtitleForRollButtonTransformer: NSValueTransformer {}
@end
@implementation DXtitleForRollButtonTransformer
+ (Class)transformedValueClass { return [NSString class]; }
+ (BOOL)allowsReverseTransformation { return NO; }
- (id)transformedValue:(id)value {
    return (value == nil) ? nil : NSStringFromClass([value class]);
}
@end*/
